# This is an auto-generated Django model module.
# You'll have to do the following manually to clean this up:
#   * Rearrange models' order
#   * Make sure each model has one field with primary_key=True
#   * Remove `managed = False` lines if you wish to allow Django to create, modify, and delete the table
# Feel free to rename the models, but don't rename db_table values or field names.
#
# Also note: You'll have to insert the output of 'django-admin.py sqlcustom [app_label]'
# into your database.


from django.db import models

from gkutils.commonutils import FLAGS, PROCESSING_FLAGS, getFlagDefs, ra_to_sex, dec_to_sex, getDateFractionMJD


class TcsDetectionLists(models.Model):
    """TcsDetectionLists.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    name = models.CharField(max_length=20)
    description = models.CharField(max_length=80, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_detection_lists'


class TcsImageGroups(models.Model):
    """TcsImageGroups.
    """

    id = models.BigIntegerField(primary_key=True)
    name = models.CharField(unique=True, max_length=255)
    group_type = models.IntegerField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_image_groups'


class AtlasMetadata(models.Model):
    """AtlasMetadata.
    """

    id = models.BigIntegerField(primary_key=True)
    filename = models.CharField(max_length=255, blank=True)
    expname = models.CharField(max_length=255, blank=True)
    object = models.CharField(max_length=255, blank=True)
    mjd_obs = models.FloatField(blank=True, null=True)
    filter = models.CharField(max_length=10, blank=True)
    pa = models.FloatField(blank=True, null=True)
    exptime = models.FloatField(blank=True, null=True)
    nx = models.IntegerField(blank=True, null=True)
    ny = models.IntegerField(blank=True, null=True)
    rad = models.IntegerField(blank=True, null=True)
    scale = models.FloatField(blank=True, null=True)
    longitude = models.FloatField(blank=True, null=True)
    latitude = models.FloatField(blank=True, null=True)
    elev = models.FloatField(blank=True, null=True)
    ra = models.FloatField(blank=True, null=True)
    dec = models.FloatField(blank=True, null=True)
    rarms = models.FloatField(blank=True, null=True)
    decrms = models.FloatField(blank=True, null=True)
    zp = models.FloatField(blank=True, null=True)
    skymag = models.FloatField(blank=True, null=True)
    cloud = models.FloatField(blank=True, null=True)
    mag5sig = models.FloatField(blank=True, null=True)
    az = models.FloatField(blank=True, null=True)
    alt = models.FloatField(blank=True, null=True)
    lambda_field = models.FloatField(db_column='lambda', blank=True, null=True)  # Field renamed because it was a Python reserved word.
    beta = models.FloatField(blank=True, null=True)
    sunelong = models.FloatField(blank=True, null=True)
    htm16id = models.BigIntegerField(db_column='htm16ID', blank=True, null=True)  # Field name made lowercase.
    input = models.CharField(max_length=255, blank=True)
    reference = models.CharField(max_length=255, blank=True)
    date_inserted = models.DateTimeField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_metadata'


# 2018-09-20 KWS Added 5 new columns, apfit, bckgnd, gain, psfphi, ddcver.
class AtlasMetadataddc(models.Model):
    """AtlasMetadataddc.
    """

    id = models.BigIntegerField(primary_key=True)
    filename = models.CharField(max_length=255, blank=True, null=True)
    obs = models.CharField(unique=True, max_length=60, blank=True, null=True)
    obj = models.CharField(max_length=60, blank=True, null=True)
    filt = models.CharField(max_length=10, blank=True, null=True)
    mjd = models.FloatField(blank=True, null=True)
    texp = models.FloatField(blank=True, null=True)
    ra = models.FloatField(blank=True, null=True)
    dec = models.FloatField(blank=True, null=True)
    pa = models.FloatField(blank=True, null=True)
    nx = models.SmallIntegerField(blank=True, null=True)
    ny = models.SmallIntegerField(blank=True, null=True)
    rad = models.IntegerField(blank=True, null=True)
    fwmaj = models.FloatField(blank=True, null=True)
    fwmin = models.FloatField(blank=True, null=True)
    psfpa = models.FloatField(blank=True, null=True)
    scale = models.FloatField(blank=True, null=True)
    long = models.FloatField(blank=True, null=True)
    lat = models.FloatField(blank=True, null=True)
    elev = models.FloatField(blank=True, null=True)
    rarms = models.FloatField(blank=True, null=True)
    decrms = models.FloatField(blank=True, null=True)
    magzpt = models.FloatField(blank=True, null=True)
    skymag = models.FloatField(blank=True, null=True)
    cloud = models.FloatField(blank=True, null=True)
    mag5sig = models.FloatField(blank=True, null=True)
    az = models.FloatField(blank=True, null=True)
    alt = models.FloatField(blank=True, null=True)
    lambda_field = models.FloatField(db_column='lambda', blank=True, null=True)  # Field renamed because it was a Python reserved word.
    beta = models.FloatField(blank=True, null=True)
    sunelong = models.FloatField(blank=True, null=True)
    apfit = models.FloatField(blank=True, null=True)
    bckgnd = models.IntegerField(blank=True, null=True)
    gain = models.FloatField(blank=True, null=True)
    psfphi = models.FloatField(blank=True, null=True)
    ddcver = models.CharField(max_length=30, blank=True, null=True)
    input = models.CharField(max_length=255, blank=True, null=True)
    reference = models.CharField(max_length=255, blank=True, null=True)
    htm16id = models.BigIntegerField(db_column='htm16ID', blank=True, null=True)  # Field name made lowercase.
    date_inserted = models.DateTimeField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_metadataddc'
        unique_together = (('filename', 'mjd'),)


class TcsImages(models.Model):
    """TcsImages.
    """

    id = models.BigIntegerField(primary_key=True)
    target = models.CharField(max_length=255)
    ref = models.CharField(max_length=255, blank=True)
    diff = models.CharField(max_length=255, blank=True)
    mjd_obs = models.FloatField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_images'

    @property
    def whole_mjd(self):
        """whole_mjd.
        """
        m = int(self.mjd_obs)
        return m


class AtlasDiffObjects(models.Model):
    """AtlasDiffObjects.
    """

    id = models.BigIntegerField(primary_key=True)
    followup_id = models.IntegerField(blank=True, null=True)
    followup_flag_date = models.DateField(blank=True, null=True)
    #detection_id = models.IntegerField()
    # Note the quoted model name in the foreign key. This model has not yet been defined.
    detection_id = models.ForeignKey('AtlasDiffDetections', to_field='id', db_column='detection_id', on_delete=models.CASCADE)
    ra = models.FloatField()
    dec = models.FloatField()
    object_classification = models.IntegerField(blank=True, null=True)
    sherlockClassification = models.CharField(max_length=120, blank=True)
    observation_status = models.CharField(max_length=40, blank=True)
    htm16id = models.BigIntegerField(db_column='htm16ID')  # Field name made lowercase.
    jtindex = models.IntegerField(blank=True, null=True)
    images_id = models.ForeignKey(TcsImages, to_field='id', db_column='images_id', on_delete=models.CASCADE)
    date_inserted = models.DateTimeField()
    date_modified = models.DateTimeField(blank=True, null=True)
    processing_flags = models.IntegerField(blank=True, null=True)
    updated_by = models.CharField(max_length=40, blank=True)
    followup_priority = models.IntegerField(blank=True, null=True)
    external_reference_id = models.CharField(max_length=40, blank=True)
    detection_list_id = models.ForeignKey(TcsDetectionLists, null=True, to_field='id', db_column='detection_list_id', on_delete=models.CASCADE)
    survey_field = models.CharField(max_length=10, blank=True)
    followup_counter = models.IntegerField(blank=True, null=True)
    atlas_designation = models.CharField(max_length=40, blank=True)
    other_designation = models.CharField(max_length=40, blank=True)
    current_trend = models.CharField(max_length=40, blank=True)
    local_comments = models.CharField(max_length=765, blank=True)
    ndetections = models.IntegerField(blank=True, null=True)
    realbogus_factor = models.FloatField(blank=True, null=True)
    zooniverse_score = models.FloatField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_diff_objects'

    @property
    def decode_flag_bits(self):
        """decode_flag_bits.
        """
        object_definition = getFlagDefs(self.object_classification, FLAGS, delimiter = ' ')
        return object_definition

    @property
    def decode_processing_flags_bits(self):
        """decode_processing_flags_bits.
        """
        pf = getFlagDefs(self.processing_flags, PROCESSING_FLAGS, delimiter = ' ')
        return pf

    @property
    def ra_sex(self):
        """ra_sex.
        """
        ra_in_sex = ra_to_sex (self.ra)
        return ra_in_sex

    @property
    def dec_sex(self):
        """dec_sex.
        """
        dec_in_sex = dec_to_sex (self.dec)
        return dec_in_sex


class AtlasDiffDetections(models.Model):
    """AtlasDiffDetections.
    """

    id = models.BigIntegerField(primary_key=True)
    atlas_metadata_id = models.ForeignKey(AtlasMetadata, to_field='id', db_column='atlas_metadata_id', on_delete=models.CASCADE)
    atlas_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='atlas_object_id', on_delete=models.CASCADE)
    tphot_id = models.IntegerField()
    x = models.FloatField(blank=True, null=True)
    y = models.FloatField(blank=True, null=True)
    peakval = models.FloatField(blank=True, null=True)
    skyval = models.FloatField(blank=True, null=True)
    peakfit = models.FloatField(blank=True, null=True)
    dpeak = models.FloatField(blank=True, null=True)
    skyfit = models.FloatField(blank=True, null=True)
    flux = models.FloatField(blank=True, null=True)
    dflux = models.FloatField(blank=True, null=True)
    major = models.FloatField(blank=True, null=True)
    minor = models.FloatField(blank=True, null=True)
    phi = models.FloatField(blank=True, null=True)
    err = models.FloatField(blank=True, null=True)
    chi_n = models.FloatField(db_column='chi_N', blank=True, null=True)  # Field name made lowercase.
    ra = models.FloatField()
    dec = models.FloatField()
    mag = models.FloatField(blank=True, null=True)
    dm = models.FloatField(blank=True, null=True)
    peak = models.FloatField(blank=True, null=True)
    sky = models.FloatField(blank=True, null=True)
    varkrn = models.FloatField(blank=True, null=True)
    pstar = models.FloatField(blank=True, null=True)
    pkast = models.FloatField(blank=True, null=True)
    preal = models.FloatField(blank=True, null=True)
    star = models.IntegerField(blank=True, null=True)
    dstar = models.IntegerField(blank=True, null=True)
    mstar = models.FloatField(blank=True, null=True)
    kast = models.IntegerField(blank=True, null=True)
    dkast = models.IntegerField(blank=True, null=True)
    htm16id = models.BigIntegerField(db_column='htm16ID')  # Field name made lowercase.
    jtindex = models.IntegerField(blank=True, null=True)
    date_inserted = models.DateTimeField()
    date_modified = models.DateTimeField(blank=True, null=True)
    image_group_id = models.ForeignKey(TcsImageGroups, to_field='id', db_column='image_group_id', on_delete=models.CASCADE)
    quality_threshold_pass = models.IntegerField(blank=True, null=True)
    deprecated = models.IntegerField(blank=True, null=True)
    realbogus_factor = models.FloatField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_diff_detections'


class AtlasDetectionsddc(models.Model):
    """AtlasDetectionsddc.
    """

    id = models.BigIntegerField(primary_key=True)
    atlas_metadata_id = models.ForeignKey(AtlasMetadataddc, to_field='id', db_column='atlas_metadata_id', on_delete=models.CASCADE)
    atlas_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='atlas_object_id', on_delete=models.CASCADE)
    det_id = models.IntegerField()
    ra = models.FloatField(blank=True, null=True)
    dec = models.FloatField(blank=True, null=True)
    mag = models.FloatField(blank=True, null=True)
    dmag = models.FloatField(blank=True, null=True)
    x = models.FloatField(blank=True, null=True)
    y = models.FloatField(blank=True, null=True)
    major = models.FloatField(blank=True, null=True)
    minor = models.FloatField(blank=True, null=True)
    phi = models.FloatField(blank=True, null=True)
    det = models.IntegerField(blank=True, null=True)
    chin = models.FloatField(blank=True, null=True)
    pvr = models.SmallIntegerField(blank=True, null=True)
    ptr = models.SmallIntegerField(blank=True, null=True)
    pmv = models.SmallIntegerField(blank=True, null=True)
    pkn = models.SmallIntegerField(blank=True, null=True)
    pno = models.SmallIntegerField(blank=True, null=True)
    pbn = models.SmallIntegerField(blank=True, null=True)
    pcr = models.SmallIntegerField(blank=True, null=True)
    pxt = models.SmallIntegerField(blank=True, null=True)
    psc = models.SmallIntegerField(blank=True, null=True)
    dup = models.IntegerField(blank=True, null=True)
    wpflx = models.FloatField(blank=True, null=True)
    dflx = models.FloatField(blank=True, null=True)
    date_modified = models.DateTimeField(blank=True, null=True)
    image_group_id = models.BigIntegerField(blank=True, null=True)
    quality_threshold_pass = models.IntegerField(blank=True, null=True)
    deprecated = models.IntegerField(blank=True, null=True)
    realbogus_factor = models.FloatField(blank=True, null=True)
    htm16id = models.BigIntegerField(db_column='htm16ID', blank=True, null=True)  # Field name made lowercase.
    date_inserted = models.DateTimeField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_detectionsddc'



# 2018-01-30 KWS Added major and minor
# 2018-09-21 KWS Added apfit
# 2018-09-25 KWS Added foreign key to AtlasMetadataddc so we can pull out
#                the correct exposure time and apfit values.
class AtlasForcedPhotometry(models.Model):
    """AtlasForcedPhotometry.
    """

    id = models.BigIntegerField(primary_key=True)
    atlas_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='atlas_object_id', on_delete=models.CASCADE)
    expname = models.ForeignKey(AtlasMetadataddc, to_field='obs', db_column='expname', on_delete=models.CASCADE)
    #expname = models.CharField(max_length=255, blank=True)
    mjd_obs = models.FloatField(blank=True, null=True)
    ra = models.FloatField(blank=True, null=True)
    dec = models.FloatField(blank=True, null=True)
    filter = models.CharField(max_length=10, blank=True)
    mag = models.FloatField(blank=True, null=True)
    dm = models.FloatField(blank=True, null=True)
    snr = models.FloatField(blank=True, null=True)
    zp = models.FloatField(blank=True, null=True)
    x = models.FloatField(blank=True, null=True)
    y = models.FloatField(blank=True, null=True)
    peakval = models.FloatField(blank=True, null=True)
    skyval = models.FloatField(blank=True, null=True)
    peakfit = models.FloatField(blank=True, null=True)
    dpeak = models.FloatField(blank=True, null=True)
    skyfit = models.FloatField(blank=True, null=True)
    flux = models.FloatField(blank=True, null=True)
    dflux = models.FloatField(blank=True, null=True)
    dflux = models.FloatField(blank=True, null=True)
    chin = models.FloatField(blank=True, null=True)
    major = models.FloatField(blank=True, null=True)
    minor = models.FloatField(blank=True, null=True)
    snrdet = models.FloatField(blank=True, null=True)
    snrlimit = models.FloatField(blank=True, null=True)
    apfit = models.FloatField(blank=True, null=True)
    date_inserted = models.DateTimeField()
    limiting_mag = models.BooleanField(null=True, blank=True)
    redo_photometry = models.BooleanField(null=True, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_forced_photometry'


class TcsCatalogueTables(models.Model):
    """TcsCatalogueTables.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    table_name = models.CharField(max_length=40)
    description = models.CharField(max_length=60)
    url = models.CharField(max_length=255, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_catalogue_tables'


class TcsClassificationFlags(models.Model):
    """TcsClassificationFlags.
    """

    flag_id = models.IntegerField(primary_key=True)
    flag_name = models.CharField(max_length=30)
    description = models.CharField(max_length=80, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_classification_flags'


class TcsClassificationHistory(models.Model):
    """TcsClassificationHistory.
    """

    id = models.BigIntegerField(primary_key=True)
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    last_classification = models.IntegerField()
    reclassification_date = models.DateTimeField()
    comments = models.CharField(max_length=255, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_classification_history'


class TcsCrossMatches(models.Model):
    """TcsCrossMatches.
    """

    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    catalogue_object_id = models.CharField(max_length=30)
    catalogue_table_id = models.IntegerField()
    search_parameters_id = models.IntegerField()
    separation = models.FloatField(blank=True, null=True)
    id = models.BigIntegerField(primary_key=True)
    z = models.FloatField(blank=True, null=True)
    scale = models.FloatField(blank=True, null=True)
    distance = models.FloatField(blank=True, null=True)
    distance_modulus = models.FloatField(blank=True, null=True)
    association_type = models.IntegerField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_cross_matches'


class TcsCrossMatchesExternal(models.Model):
    """TcsCrossMatchesExternal.
    """

    id = models.BigIntegerField(primary_key=True)
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    external_designation = models.CharField(max_length=60)
    type = models.CharField(max_length=40, blank=True)
    host_galaxy = models.CharField(max_length=60, blank=True)
    mag = models.FloatField(blank=True, null=True)
    discoverer = models.CharField(max_length=300, blank=True)
    matched_list = models.CharField(max_length=100)
    other_info = models.CharField(max_length=300, blank=True)
    separation = models.FloatField(blank=True, null=True)
    comments = models.CharField(max_length=300, blank=True)
    url = models.CharField(max_length=300, blank=True)
    host_z = models.FloatField(blank=True, null=True)
    object_z = models.FloatField(blank=True, null=True)
    disc_date = models.DateTimeField(blank=True, null=True)
    disc_filter = models.CharField(max_length=150, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_cross_matches_external'


class TcsFollowupPhotometry(models.Model):
    """TcsFollowupPhotometry.
    """

    id = models.BigIntegerField(primary_key=True)
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    telescope_instrument_id = models.IntegerField()
    mjd = models.FloatField()
    filter = models.CharField(max_length=20, blank=True)
    mag = models.FloatField(blank=True, null=True)
    magerr = models.FloatField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_followup_photometry'


class TcsFollowupTelescopeInstruments(models.Model):
    """TcsFollowupTelescopeInstruments.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    telescope_id = models.IntegerField()
    name = models.CharField(max_length=30)
    description = models.CharField(max_length=60)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_followup_telescope_instruments'


class TcsFollowupTelescopes(models.Model):
    """TcsFollowupTelescopes.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    name = models.CharField(max_length=30)
    description = models.CharField(max_length=60)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_followup_telescopes'


class TcsForcedPhotometry(models.Model):
    """TcsForcedPhotometry.
    """

    id = models.BigIntegerField(primary_key=True)
    postage_stamp_request_id = models.BigIntegerField(blank=True, null=True)
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    rownum = models.IntegerField(blank=True, null=True)
    skycell = models.CharField(max_length=10, blank=True)
    mjd_obs = models.FloatField(blank=True, null=True)
    exptime = models.FloatField(blank=True, null=True)
    filter = models.CharField(max_length=20, blank=True)
    fpa_id = models.CharField(max_length=20, blank=True)
    x_psf = models.FloatField(blank=True, null=True)
    y_psf = models.FloatField(blank=True, null=True)
    x_psf_sig = models.FloatField(blank=True, null=True)
    y_psf_sig = models.FloatField(blank=True, null=True)
    posangle = models.FloatField(blank=True, null=True)
    pltscale = models.FloatField(blank=True, null=True)
    psf_inst_mag = models.FloatField(blank=True, null=True)
    psf_inst_mag_sig = models.FloatField(blank=True, null=True)
    psf_inst_flux = models.FloatField(blank=True, null=True)
    psf_inst_flux_sig = models.FloatField(blank=True, null=True)
    ap_mag = models.FloatField(blank=True, null=True)
    ap_mag_radius = models.FloatField(blank=True, null=True)
    peak_flux_as_mag = models.FloatField(blank=True, null=True)
    cal_psf_mag = models.FloatField(blank=True, null=True)
    cal_psf_mag_sig = models.FloatField(blank=True, null=True)
    ra_psf = models.FloatField()
    dec_psf = models.FloatField()
    sky = models.FloatField(blank=True, null=True)
    sky_sigma = models.FloatField(blank=True, null=True)
    psf_chisq = models.FloatField(blank=True, null=True)
    cr_nsigma = models.FloatField(blank=True, null=True)
    ext_nsigma = models.FloatField(blank=True, null=True)
    psf_major = models.FloatField(blank=True, null=True)
    psf_minor = models.FloatField(blank=True, null=True)
    psf_theta = models.FloatField(blank=True, null=True)
    psf_qf = models.FloatField(blank=True, null=True)
    psf_ndof = models.IntegerField(blank=True, null=True)
    psf_npix = models.IntegerField(blank=True, null=True)
    moments_xx = models.FloatField(blank=True, null=True)
    moments_xy = models.FloatField(blank=True, null=True)
    moments_yy = models.FloatField(blank=True, null=True)
    diff_npos = models.IntegerField(blank=True, null=True)
    diff_fratio = models.FloatField(blank=True, null=True)
    diff_nratio_bad = models.FloatField(blank=True, null=True)
    diff_nratio_mask = models.FloatField(blank=True, null=True)
    diff_nratio_all = models.FloatField(blank=True, null=True)
    flags = models.IntegerField(blank=True, null=True)
    n_frames = models.IntegerField(blank=True, null=True)
    padding = models.IntegerField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_forced_photometry'


# 2015-11-23 KWS Experiment with OneToOneField for id, since this is both
#                a primary key and a foreign key.
class TcsLatestObjectStats(models.Model):
    """TcsLatestObjectStats.
    """

    #id = models.OneToOneField(AtlasDiffObjects, on_delete=models.CASCADE, db_column='id', related_name='stats', primary_key=True, blank=True, null=True)
    id = models.OneToOneField(AtlasDiffObjects, on_delete=models.CASCADE, db_column='id', related_name='stats', primary_key=True)
    latest_mjd = models.FloatField(blank=True, null=True)
    latest_mag = models.FloatField(blank=True, null=True)
    latest_filter = models.CharField(max_length=80, blank=True)
    earliest_mjd = models.FloatField(blank=True, null=True)
    earliest_mag = models.FloatField(blank=True, null=True)
    earliest_filter = models.CharField(max_length=80, blank=True)
    ra_avg = models.FloatField(blank=True, null=True)
    dec_avg = models.FloatField(blank=True, null=True)
    catalogue = models.CharField(max_length=60, blank=True)
    catalogue_object_id = models.CharField(max_length=30, blank=True)
    separation = models.FloatField(blank=True, null=True)
    external_crossmatches = models.CharField(max_length=330, blank=True)
    discovery_target = models.CharField(max_length=80, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_latest_object_stats'

    @property
    def ra_sex(self):
        """ra_sex.
        """
        ra_in_sex = ra_to_sex (self.ra_avg)
        return ra_in_sex

    @property
    def dec_sex(self):
        """dec_sex.
        """
        dec_in_sex = dec_to_sex (self.dec_avg)
        return dec_in_sex

    @property
    def earliest_mjd_date(self):
        """earliest_mjd_date.
        """
        dateFraction = getDateFractionMJD(self.earliest_mjd, delimiter = '-')
        return dateFraction

    @property
    def latest_mjd_date(self):
        """latest_mjd_date.
        """
        dateFraction = getDateFractionMJD(self.latest_mjd, delimiter = '-')
        return dateFraction


class TcsObjectGroupDefinitions(models.Model):
    """TcsObjectGroupDefinitions.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    name = models.CharField(max_length=20)
    description = models.CharField(max_length=80, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_object_group_definitions'


class TcsObjectGroups(models.Model):
    """TcsObjectGroups.
    """

    id = models.BigIntegerField(primary_key=True)
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    object_group_id = models.IntegerField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_object_groups'


class TcsParameterDefinitions(models.Model):
    """TcsParameterDefinitions.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    name = models.CharField(max_length=40)
    type = models.CharField(max_length=20)
    value = models.CharField(max_length=40)
    units = models.CharField(max_length=40)
    catalogue_table_id = models.IntegerField()
    search_parameters_id = models.IntegerField()

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_parameter_definitions'


class TcsPostageStampImages(models.Model):
    """TcsPostageStampImages.
    """

    id = models.BigIntegerField(primary_key=True)
    image_type = models.CharField(max_length=20)
    image_filename = models.CharField(max_length=255)
    pss_filename = models.CharField(max_length=255, blank=True)
    mjd_obs = models.FloatField(blank=True, null=True)
    image_group_id = models.ForeignKey(TcsImageGroups, to_field='id', db_column='image_group_id', on_delete=models.CASCADE)
    pss_error_code = models.IntegerField()
    filter = models.CharField(max_length=80, blank=True)
    mask_ratio = models.FloatField(blank=True, null=True)
    mask_ratio_at_core = models.FloatField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_postage_stamp_images'


    @property
    def filename(self):
        """filename.
        """
        mjdstring = int(float(self.image_filename.split('_')[1]))
        f = str(mjdstring) + '/' + self.image_filename
        return f

    @property
    def stamp_mjd(self):
        """stamp_mjd.
        """
        return self.mjd_obs

    @property
    def whole_mjd(self):
        """whole_mjd.
        """
        # The second field is always the target MJD
        fields = self.image_filename.split('_')
        m = int(float(fields[1]))
        return m

class TcsPostageStampRequests(models.Model):
    """TcsPostageStampRequests.
    """

    id = models.BigIntegerField(primary_key=True)
    name = models.CharField(unique=True, max_length=80)
    pss_id = models.BigIntegerField(blank=True, null=True)
    download_attempts = models.IntegerField()
    status = models.IntegerField()
    created = models.DateTimeField()
    updated = models.DateTimeField(blank=True, null=True)
    request_type = models.IntegerField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_postage_stamp_requests'


class TcsPostageStampStatusCodes(models.Model):
    """TcsPostageStampStatusCodes.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    code = models.CharField(max_length=20)
    description = models.CharField(max_length=80, blank=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_postage_stamp_status_codes'


class TcsProcessingStatus(models.Model):
    """TcsProcessingStatus.
    """

    id = models.BigIntegerField(primary_key=True)
    status = models.IntegerField()
    started = models.DateTimeField()
    modified = models.DateTimeField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_processing_status'


class TcsSearchParameters(models.Model):
    """TcsSearchParameters.
    """

    id = models.IntegerField(primary_key=True)  # AutoField?
    parameter_set_name = models.CharField(max_length=20)
    comments = models.CharField(max_length=80, blank=True)
    last_updated = models.DateTimeField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_search_parameters'


class TcsZooRequests(models.Model):
    """TcsZooRequests.
    """

    id = models.BigIntegerField(primary_key=True)
    name = models.CharField(unique=True, max_length=80)
    zoo_id = models.BigIntegerField(blank=True, null=True)
    download_attempts = models.IntegerField()
    status = models.IntegerField()
    created = models.DateTimeField()
    updated = models.DateTimeField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'tcs_zoo_requests'

# 2017-03-21 KWS Four new database tables for Sherlock and Gravitational Waves.
# 2017-06-20 KWS Dave has updated the definition of sherlock_classifications.
class SherlockClassifications(models.Model):
    """SherlockClassifications.
    """

    transient_object_id = models.OneToOneField(AtlasDiffObjects, to_field='id', db_column='transient_object_id', primary_key=True, on_delete=models.CASCADE)
    classification = models.CharField(max_length=45, blank=True, null=True)
    annotation = models.TextField(blank=True, null=True)
    summary = models.CharField(max_length=50, blank=True, null=True)
    matchverified = models.IntegerField(db_column='matchVerified', blank=True, null=True)  # Field name made lowercase.
    developmentcomment = models.CharField(db_column='developmentComment', max_length=100, blank=True, null=True)  # Field name made lowercase.
    datelastmodified = models.DateTimeField(db_column='dateLastModified', blank=True, null=True)  # Field name made lowercase.
    datecreated = models.DateTimeField(db_column='dateCreated', blank=True, null=True)  # Field name made lowercase.
    updated = models.CharField(max_length=45, blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'sherlock_classifications'


# 2017-06-20 KWS Dave has updated the definition of sherlock_crossmatches.
class SherlockCrossmatches(models.Model):
    """SherlockCrossmatches.
    """

    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    catalogue_object_id = models.CharField(max_length=30, blank=True, null=True)
    catalogue_table_id = models.SmallIntegerField(blank=True, null=True)
    separationarcsec = models.FloatField(db_column='separationArcsec', blank=True, null=True)  # Field name made lowercase.
    northseparationarcsec = models.FloatField(db_column='northSeparationArcsec', blank=True, null=True)  # Field name made lowercase.
    eastseparationarcsec = models.FloatField(db_column='eastSeparationArcsec', blank=True, null=True)  # Field name made lowercase.
    id = models.BigIntegerField(primary_key=True)
    z = models.FloatField(blank=True, null=True)
    scale = models.FloatField(blank=True, null=True)
    distance = models.FloatField(blank=True, null=True)
    distance_modulus = models.FloatField(blank=True, null=True)
    photoz = models.FloatField(db_column='photoZ', blank=True, null=True)  # Field name made lowercase.
    photozerr = models.FloatField(db_column='photoZErr', blank=True, null=True)  # Field name made lowercase.
    association_type = models.CharField(max_length=45, blank=True, null=True)
    datecreated = models.DateTimeField(db_column='dateCreated', blank=True, null=True)  # Field name made lowercase.
    physical_separation_kpc = models.FloatField(blank=True, null=True)
    catalogue_object_type = models.CharField(max_length=45, blank=True, null=True)
    catalogue_object_subtype = models.CharField(max_length=45, blank=True, null=True)
    association_rank = models.IntegerField(blank=True, null=True)
    catalogue_table_name = models.CharField(max_length=100, blank=True, null=True)
    catalogue_view_name = models.CharField(max_length=100, blank=True, null=True)
    rank = models.IntegerField(blank=True, null=True)
    rankscore = models.FloatField(db_column='rankScore', blank=True, null=True)  # Field name made lowercase.
    search_name = models.CharField(max_length=100, blank=True, null=True)
    major_axis_arcsec = models.FloatField(blank=True, null=True)
    direct_distance = models.FloatField(blank=True, null=True)
    direct_distance_scale = models.FloatField(blank=True, null=True)
    direct_distance_modulus = models.FloatField(blank=True, null=True)
    radeg = models.FloatField(db_column='raDeg', blank=True, null=True)  # Field name made lowercase.
    decdeg = models.FloatField(db_column='decDeg', blank=True, null=True)  # Field name made lowercase.
    original_search_radius_arcsec = models.FloatField(blank=True, null=True)
    catalogue_view_id = models.IntegerField(blank=True, null=True)
    u = models.FloatField(db_column='U', blank=True, null=True)  # Field name made lowercase.
    uerr = models.FloatField(db_column='UErr', blank=True, null=True)  # Field name made lowercase.
    b = models.FloatField(db_column='B', blank=True, null=True)  # Field name made lowercase.
    berr = models.FloatField(db_column='BErr', blank=True, null=True)  # Field name made lowercase.
    v = models.FloatField(db_column='V', blank=True, null=True)  # Field name made lowercase.
    verr = models.FloatField(db_column='VErr', blank=True, null=True)  # Field name made lowercase.
    r = models.FloatField(db_column='R', blank=True, null=True)  # Field name made lowercase.
    rerr = models.FloatField(db_column='RErr', blank=True, null=True)  # Field name made lowercase.
    i = models.FloatField(db_column='I', blank=True, null=True)  # Field name made lowercase.
    ierr = models.FloatField(db_column='IErr', blank=True, null=True)  # Field name made lowercase.
    j = models.FloatField(db_column='J', blank=True, null=True)  # Field name made lowercase.
    jerr = models.FloatField(db_column='JErr', blank=True, null=True)  # Field name made lowercase.
    h = models.FloatField(db_column='H', blank=True, null=True)  # Field name made lowercase.
    herr = models.FloatField(db_column='HErr', blank=True, null=True)  # Field name made lowercase.
    k = models.FloatField(db_column='K', blank=True, null=True)  # Field name made lowercase.
    kerr = models.FloatField(db_column='KErr', blank=True, null=True)  # Field name made lowercase.
    field_u = models.FloatField(db_column='_u', blank=True, null=True)  # Field renamed because it started with '_'.
    field_uerr = models.FloatField(db_column='_uErr', blank=True, null=True)  # Field name made lowercase. Field renamed because it started with '_'.
    field_g = models.FloatField(db_column='_g', blank=True, null=True)  # Field renamed because it started with '_'.
    field_gerr = models.FloatField(db_column='_gErr', blank=True, null=True)  # Field name made lowercase. Field renamed because it started with '_'.
    field_r = models.FloatField(db_column='_r', blank=True, null=True)  # Field renamed because it started with '_'.
    field_rerr = models.FloatField(db_column='_rErr', blank=True, null=True)  # Field name made lowercase. Field renamed because it started with '_'.
    field_i = models.FloatField(db_column='_i', blank=True, null=True)  # Field renamed because it started with '_'.
    field_ierr = models.FloatField(db_column='_iErr', blank=True, null=True)  # Field name made lowercase. Field renamed because it started with '_'.
    field_z = models.FloatField(db_column='_z', blank=True, null=True)  # Field renamed because it started with '_'.
    field_zerr = models.FloatField(db_column='_zErr', blank=True, null=True)  # Field name made lowercase. Field renamed because it started with '_'.
    field_y = models.FloatField(db_column='_y', blank=True, null=True)  # Field renamed because it started with '_'.
    field_yerr = models.FloatField(db_column='_yErr', blank=True, null=True)  # Field name made lowercase. Field renamed because it started with '_'.
    g = models.FloatField(db_column='G', blank=True, null=True)  # Field name made lowercase.
    gerr = models.FloatField(db_column='GErr', blank=True, null=True)  # Field name made lowercase.
    unkmag = models.FloatField(db_column='unkMag', blank=True, null=True)  # Field name made lowercase.
    unkmagerr = models.FloatField(db_column='unkMagErr', blank=True, null=True)  # Field name made lowercase.
    datelastmodified = models.DateTimeField(db_column='dateLastModified', blank=True, null=True)  # Field name made lowercase.
    updated = models.IntegerField(blank=True, null=True)
    classificationreliability = models.IntegerField(db_column='classificationReliability', blank=True, null=True)  # Field name made lowercase.
    transientabsmag = models.FloatField(db_column='transientAbsMag', blank=True, null=True)  # Field name made lowercase.

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'sherlock_crossmatches'


class TcsGravityEvents(models.Model):
    """TcsGravityEvents.
    """

    gravity_event_id = models.CharField(primary_key=True, max_length=10)
    gracedb_id = models.CharField(max_length=10)
    mjd = models.FloatField(blank=True, null=True)
    datelastmodified = models.DateTimeField(db_column='dateLastModified', blank=True, null=True)  # Field name made lowercase.
    updated = models.IntegerField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        db_table = 'tcs_gravity_events'


class TcsGravityEventAnnotations(models.Model):
    """TcsGravityEventAnnotations.
    """

    primaryid = models.BigIntegerField(db_column='primaryId', primary_key=True)  # Field name made lowercase.
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    gravity_event_id = models.ForeignKey(TcsGravityEvents, to_field='gravity_event_id', db_column='gravity_event_id', on_delete=models.CASCADE)
    gracedb_id = models.CharField(max_length=10)
    enclosing_contour = models.IntegerField(blank=True, null=True)
    map_name = models.CharField(max_length=30, blank=True, null=True)
    datelastmodified = models.DateTimeField(db_column='dateLastModified', blank=True, null=True)  # Field name made lowercase.
    updated = models.IntegerField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        db_table = 'tcs_gravity_event_annotations'
        unique_together = (('transient_object_id', 'gracedb_id'),)

# 2017-05-09 KWS Added TcsObjectComments table. Comments will be inserted here in future.
class TcsObjectComments(models.Model):
    """TcsObjectComments.
    """

    id = models.BigIntegerField(db_column='id', primary_key=True)
    transient_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='transient_object_id', on_delete=models.CASCADE)
    date_inserted = models.DateTimeField(db_column='date_inserted', blank=False, null=False)
    comment = models.CharField(max_length=768, db_column='comment', blank=True, null=True)
    username = models.CharField(max_length=90, db_column='username', blank=True, null=True)

    class Meta:
        """Meta.
        """

        db_table = 'tcs_object_comments'


# 2019-06-06 KWS Added AtlasStackedForcedPhotometry table.
class AtlasStackedForcedPhotometry(models.Model):
    """AtlasStackedForcedPhotometry.
    """

    id = models.BigIntegerField(primary_key=True)
    atlas_object_id = models.ForeignKey(AtlasDiffObjects, to_field='id', db_column='atlas_object_id', on_delete=models.CASCADE)
    mjd = models.FloatField(blank=True, null=True)
    m = models.FloatField(blank=True, null=True)
    dm = models.FloatField(blank=True, null=True)
    ujy = models.FloatField(blank=True, null=True)
    dujy = models.FloatField(blank=True, null=True)
    f = models.CharField(max_length=10, blank=True, null=True)
    err = models.FloatField(blank=True, null=True)
    chin = models.FloatField(blank=True, null=True)
    ra = models.FloatField(blank=True, null=True)
    dec = models.FloatField(blank=True, null=True)
    x = models.FloatField(blank=True, null=True)
    y = models.FloatField(blank=True, null=True)
    maj = models.FloatField(blank=True, null=True)
    min = models.FloatField(blank=True, null=True)
    phi = models.FloatField(blank=True, null=True)
    apfit = models.FloatField(blank=True, null=True)
    sky = models.FloatField(blank=True, null=True)
    zp = models.FloatField(blank=True, null=True)
    stack = models.CharField(max_length=60, blank=True, null=True)
    date_inserted = models.DateTimeField()
    redo_photometry = models.IntegerField(blank=True, null=True)

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_stacked_forced_photometry'



from django.db import models
#from atlas.utils import *
from math import log10

from atlas.models import TcsCrossMatchesExternal, TcsDetectionLists, TcsImages
from gkutils.commonutils import ra_to_sex, dec_to_sex, getFlagDefs, getDateFractionMJD, FLAGS, transform, J2000toGalactic

class CustomLCPoints(models.Model):
    """CustomLCPoints.
    """

    id = models.IntegerField(primary_key=True, db_column='id')
    mag = models.FloatField(db_column='mag')
    magerr = models.FloatField(db_column='magerr')
    mjd = models.FloatField(db_column='mjd')
    exptime = models.FloatField(db_column='exptime')
    filter = models.CharField(max_length=90, db_column='filter')
    zp = models.FloatField(db_column='zp')
    expname = models.CharField(max_length=90, db_column='expname')
    ra = models.FloatField(db_column='ra')
    dec = models.FloatField(db_column='dec')
    atlas_metadata_id = models.IntegerField(db_column='atlas_metadata_id')

class CustomLCBlanks(models.Model):
    """CustomLCBlanks.
    """

    id = models.IntegerField(primary_key=True, db_column='id')
    mjd = models.FloatField(db_column='mjd')
    exptime = models.FloatField(db_column='exptime')
    filter = models.CharField(max_length=90, db_column='filter')
    zp = models.FloatField(db_column='zp')
    expname = models.CharField(max_length=90, db_column='expname')
    filename = models.CharField(max_length=765, db_column='filename')
    input = models.CharField(max_length=765, db_column='input')
    reference = models.CharField(max_length=765, db_column='reference')
    pointing = models.CharField(max_length=765, db_column='pointing')

class CustomFollowupLCData(models.Model):
    """CustomFollowupLCData.
    """

    id = models.IntegerField(primary_key=True, db_column='id')
    transient_object_id = models.IntegerField(db_column='transient_object_id')
    mjd = models.FloatField(db_column='mjd')
    mag = models.FloatField(db_column='mag')
    magerr = models.FloatField(db_column='magerr')
    filter = models.CharField(max_length=90, db_column='filter')
    telescope_name = models.CharField(max_length=90, db_column='telescope_name')
    telescope_description = models.CharField(max_length=180, db_column='telescope_description')
    instrument_name = models.CharField(max_length=90, db_column='instrument_name')
    instrument_description = models.CharField(max_length=180, db_column='instrument_description')

class FollowupRaw(models.Model):
    """FollowupRaw.
    """

    rank = models.IntegerField(db_column='rank')
    id = models.BigIntegerField(primary_key=True, db_column='id')
    atlas_designation = models.CharField(max_length=60, db_column='atlas_designation')
    other_designation = models.CharField(max_length=60, db_column='other_designation')
    ra = models.FloatField(db_column='ra')
    dec = models.FloatField(db_column='dec')
    object_classification = models.IntegerField(db_column='object_classification')
    followup_flag_date = models.DateField(db_column='followup_flag_date')
    observation_status = models.CharField(max_length=40, db_column='observation_status')
    current_trend = models.CharField(max_length=40, db_column='current_trend')
    earliest_mjd = models.FloatField(db_column='earliest_mjd')
    earliest_mag = models.FloatField(db_column='earliest_mag')
    earliest_filter = models.CharField(max_length=80, db_column='earliest_filter')
    latest_mjd = models.FloatField(db_column='latest_mjd')
    latest_mag = models.FloatField(db_column='latest_mag')
    latest_filter = models.CharField(max_length=80, db_column='latest_filter')
    catalogue = models.CharField(max_length=60, db_column='catalogue')
    catalogue_object_id = models.CharField(max_length=30, db_column='catalogue_object_id')
    separation = models.FloatField(db_column='separation')
    realbogus_factor = models.FloatField(db_column='realbogus_factor')
    zooniverse_score = models.FloatField(db_column='zooniverse_score')
    date_modified = models.FloatField(db_column='date_modified')
    external_crossmatches = models.CharField(max_length=1500, db_column='external_crossmatches')
    discovery_target = models.CharField(max_length=90, db_column='discovery_target')

    @property
    def ra_sex(self):
        """ra_sex.
        """
        ra_in_sex = ra_to_sex (self.ra)
        return ra_in_sex

    @property
    def dec_sex(self):
        """dec_sex.
        """
        dec_in_sex = dec_to_sex (self.dec)
        return dec_in_sex

    @property
    def decode_flag_bits(self):
        """decode_flag_bits.
        """
        object_definition = getFlagDefs(self.object_classification, FLAGS, delimiter = ' ')
        return object_definition

    @property
    def externalXMs(self):
        """This is a Hack to get all the external crossmatches per row. Note that
           it only gets executed 100 times (for each page) so it is not disastrous
           for database performance.
        """
        xms = TcsCrossMatchesExternal.objects.filter(transient_object_id__id=self.id).order_by('external_designation')
        #names = xms.values_list("external_designation", flat=True)
        #nameColumn = ", ".join(names)
        return xms



class WebViewAbstractFollowup(models.Model):
    """WebViewAbstractFollowup.
    """

    rank = models.IntegerField(db_column='rank')
    id = models.BigIntegerField(primary_key=True, db_column='id')
    atlas_designation = models.CharField(max_length=60, db_column='atlas_designation')
    other_designation = models.CharField(max_length=60, db_column='other_designation')
    ra = models.FloatField(db_column='ra')
    dec = models.FloatField(db_column='dec')
    ra_avg = models.FloatField(db_column='ra_avg')
    dec_avg = models.FloatField(db_column='dec_avg')
    object_classification = models.IntegerField(db_column='object_classification')
    sherlockClassification = models.CharField(max_length=120, db_column='sherlockClassification')
    followup_flag_date = models.DateField(db_column='followup_flag_date')
    observation_status = models.CharField(max_length=40, db_column='observation_status')
    current_trend = models.CharField(max_length=40, db_column='current_trend')
    earliest_mjd = models.FloatField(db_column='earliest_mjd')
    earliest_mag = models.FloatField(db_column='earliest_mag')
    earliest_filter = models.CharField(max_length=80, db_column='earliest_filter')
    latest_mjd = models.FloatField(db_column='latest_mjd')
    latest_mag = models.FloatField(db_column='latest_mag')
    latest_filter = models.CharField(max_length=80, db_column='latest_filter')
    catalogue = models.CharField(max_length=60, db_column='catalogue')
    catalogue_object_id = models.CharField(max_length=30, db_column='catalogue_object_id')
    separation = models.FloatField(db_column='separation')
    realbogus_factor = models.FloatField(db_column='realbogus_factor')
    zooniverse_score = models.FloatField(db_column='zooniverse_score')
    date_modified = models.FloatField(db_column='date_modified')
    external_crossmatches = models.CharField(max_length=1500, db_column='external_crossmatches')
    discovery_target = models.CharField(max_length=90, db_column='discovery_target')
    rms = models.FloatField(db_column='rms')
    detection_list_id = models.ForeignKey(TcsDetectionLists, null=True, to_field='id', db_column='detection_list_id', on_delete=models.CASCADE)
    images_id = models.ForeignKey(TcsImages, to_field='id', db_column='images_id', on_delete=models.CASCADE)

    class Meta:
        """Meta.
        """

        abstract = True

    @property
    def ra_sex(self):
        """ra_sex.
        """
        if self.ra_avg:
            ra_in_sex = ra_to_sex (self.ra_avg)
        else:
            ra_in_sex = ra_to_sex (self.ra)
        return ra_in_sex

    @property
    def dec_sex(self):
        """dec_sex.
        """
        if self.dec_avg:
            dec_in_sex = dec_to_sex (self.dec_avg)
        else:
            dec_in_sex = dec_to_sex (self.dec)
        return dec_in_sex

    @property
    def decode_flag_bits(self):
        """decode_flag_bits.
        """
        object_definition = getFlagDefs(self.object_classification, FLAGS, delimiter = ' ')
        return object_definition

    @property
    def externalXMs(self):
        """This is a Hack to get all the external crossmatches per row. Note that
           it only gets executed 100 times (for each page) so it is not disastrous
           for database performance.
        """
        xms = TcsCrossMatchesExternal.objects.filter(transient_object_id__id=self.id).order_by('external_designation')
        #names = xms.values_list("external_designation", flat=True)
        #nameColumn = ", ".join(names)
        #sys.stderr.write('\nOBJECT (%s) = %s\n' % (self.ID, nameColumn))
        return xms

    @property
    def galactic(self):
        """galactic.
        """
        if self.ra_avg and self.dec_avg:
            g = transform([self.ra_avg, self.dec_avg], J2000toGalactic)
        else:
            g = transform([self.ra, self.dec], J2000toGalactic)
        return g


class WebViewFollowupTransients(WebViewAbstractFollowup):
    """WebViewFollowupTransients.
    """


    class Meta(WebViewAbstractFollowup.Meta):
        """Meta.
        """

        db_table = 'atlas_v_followupall'

class WebViewFollowupTransients0(WebViewAbstractFollowup):
    """WebViewFollowupTransients0.
    """


    class Meta(WebViewAbstractFollowup.Meta):
        """Meta.
        """

        db_table = 'atlas_v_followup0'

class WebViewFollowupTransients1(WebViewAbstractFollowup):
    """WebViewFollowupTransients1.
    """


    class Meta(WebViewAbstractFollowup.Meta):
        """Meta.
        """

        db_table = 'atlas_v_followup1'

class WebViewFollowupTransients2(WebViewAbstractFollowup):
    """WebViewFollowupTransients2.
    """


    class Meta(WebViewAbstractFollowup.Meta):
        """Meta.
        """

        db_table = 'atlas_v_followup2'

class WebViewFollowupTransients3(WebViewAbstractFollowup):
    """WebViewFollowupTransients3.
    """


    class Meta(WebViewAbstractFollowup.Meta):
        """Meta.
        """

        db_table = 'atlas_v_followup3'

class WebViewFollowupTransients4(WebViewAbstractFollowup):
    """WebViewFollowupTransients4.
    """


    class Meta(WebViewAbstractFollowup.Meta):
        """Meta.
        """

        db_table = 'atlas_v_followup4'

class WebViewFollowupTransients5(WebViewAbstractFollowup):
    """WebViewFollowupTransients5.
    """


    class Meta(WebViewAbstractFollowup.Meta):
        """Meta.
        """

        db_table = 'atlas_v_followup5'

class WebViewFollowupTransients6(WebViewAbstractFollowup):
    """WebViewFollowupTransients6.
    """


    class Meta(WebViewAbstractFollowup.Meta):
        """Meta.
        """

        db_table = 'atlas_v_followup6'

class WebViewFollowupTransients7(WebViewAbstractFollowup):
    """WebViewFollowupTransients7.
    """


    class Meta(WebViewAbstractFollowup.Meta):
        """Meta.
        """

        db_table = 'atlas_v_followup7'

class WebViewFollowupTransients8(WebViewAbstractFollowup):
    """WebViewFollowupTransients8.
    """


    class Meta(WebViewAbstractFollowup.Meta):
        """Meta.
        """

        db_table = 'atlas_v_followup8'

class WebViewFollowupTransientsGeneric(WebViewAbstractFollowup):
    """WebViewFollowupTransientsGeneric.
    """


    class Meta(WebViewAbstractFollowup.Meta):
        """Meta.
        """

        db_table = 'atlas_v_followup'

class WebViewFollowupTransientsGenericGW(WebViewAbstractFollowup):
    """WebViewFollowupTransientsGenericGW.
    """


    class Meta(WebViewAbstractFollowup.Meta):
        """Meta.
        """

        db_table = 'atlas_v_followup_gw'


# 2011-04-14 KWS New model for User Defined Lists.
# 2013-10-23 KWS Added confidence_factor.
# 2014-02-20 KWS Added external_crossmatches and discovery_target.

class WebViewAbstractUserDefined(models.Model):
    """WebViewAbstractUserDefined.
    """

    rank = models.IntegerField(db_column='rank')
    id = models.BigIntegerField(db_column='id', primary_key=True)
    atlas_designation = models.CharField(max_length=60, db_column='atlas_designation')
    other_designation = models.CharField(max_length=60, db_column='other_designation')
    local_comments = models.CharField(max_length=768, db_column='local_comments')
    ra = models.FloatField(db_column='ra')
    dec = models.FloatField(db_column='dec')
    ra_avg = models.FloatField(db_column='ra_avg')
    dec_avg = models.FloatField(db_column='dec_avg')
    object_classification = models.IntegerField(db_column='object_classification')
    sherlockClassification = models.CharField(max_length=120, db_column='sherlockClassification')
    followup_flag_date = models.DateField(db_column='followup_flag_date')
    observation_status = models.CharField(max_length=40, db_column='observation_status')
    current_trend = models.CharField(max_length=40, db_column='current_trend')
    earliest_mjd = models.FloatField(db_column='earliest_mjd')
    earliest_mag = models.FloatField(db_column='earliest_mag')
    earliest_filter = models.CharField(max_length=80, db_column='earliest_filter')
    latest_mjd = models.FloatField(db_column='latest_mjd')
    latest_mag = models.FloatField(db_column='latest_mag')
    latest_filter = models.CharField(max_length=80, db_column='latest_filter')
    catalogue = models.CharField(max_length=60, db_column='catalogue')
    catalogue_object_id = models.CharField(max_length=30, db_column='catalogue_object_id')
    separation = models.FloatField(db_column='separation')
    # Extra columns for the user defined object list table
    object_group_id = models.IntegerField(db_column='object_group_id')
    detection_list_id = models.ForeignKey(TcsDetectionLists, null=True, to_field='id', db_column='detection_list_id', on_delete=models.CASCADE)
    realbogus_factor = models.FloatField(db_column='realbogus_factor')
    zooniverse_score = models.FloatField(db_column='zooniverse_score')
    date_modified = models.DateTimeField(db_column='date_modified')
    external_crossmatches = models.CharField(max_length=1500, db_column='external_crossmatches')
    discovery_target = models.CharField(max_length=90, db_column='discovery_target')
    rms = models.FloatField(db_column='rms')

    class Meta:
        """Meta.
        """

        abstract = True

    @property
    def ra_sex(self):
        """ra_sex.
        """
        ra_in_sex = ra_to_sex (self.ra)
        return ra_in_sex

    @property
    def dec_sex(self):
        """dec_sex.
        """
        dec_in_sex = dec_to_sex (self.dec)
        return dec_in_sex

    # 2013-12-20 KWS Added option to grab space-delimited RA and DEC (e.g. for producing catalogues)
    @property
    def ra_sex_spaces(self):
        """ra_sex_spaces.
        """
        ra_in_sex = ra_to_sex (self.ra, delimiter=' ')
        return ra_in_sex

    @property
    def dec_sex_spaces(self):
        """dec_sex_spaces.
        """
        dec_in_sex = dec_to_sex (self.dec, delimiter=' ')
        return dec_in_sex

    @property
    def decode_flag_bits(self):
        """decode_flag_bits.
        """
        object_definition = getFlagDefs(self.object_classification, FLAGS, delimiter = ' ')
        return object_definition

    @property
    def externalXMs(self):
        """This is a Hack to get all the external crossmatches per row. Note that
           it only gets executed 100 times (for each page) so it is not disastrous
           for database performance.
        """
        xms = TcsCrossMatchesExternal.objects.filter(transient_object_id__id=self.id).order_by('external_designation')
        #names = xms.values_list("external_designation", flat=True)
        #nameColumn = ", ".join(names)
        #sys.stderr.write('\nOBJECT (%s) = %s\n' % (self.ID, nameColumn))
        return xms

    # 2015-03-13 KWS New methods to retrieve the earliest and latest dates
    #                in date format.
    @property
    def getEarliestDate(self):
        """getEarliestDate.
        """
        dateFraction = getDateFractionMJD(self.earliest_mjd)
        return dateFraction

    @property
    def getLatestDate(self):
        """getLatestDate.
        """
        dateFraction = getDateFractionMJD(self.latest_mjd)
        return dateFraction

class WebViewUserDefined(WebViewAbstractUserDefined):
    """WebViewUserDefined.
    """


    class Meta(WebViewAbstractUserDefined.Meta):
        """Meta.
        """

        db_table = 'atlas_v_followup_userdefined'


# 2018-08-01 KWS Added custom view for getting ATLAS recurrences for PESSTO.
#                This is much faster than the ORM query and doesn't take up
#                all the memory!
class AtlasVRecurrencesddcPessto(models.Model):
    """AtlasVRecurrencesddcPessto.
    """

    rank = models.IntegerField(db_column='rank')
    id = models.BigIntegerField(db_column='id', primary_key=True)
    name = models.CharField(max_length=90, db_column='name')
    tns_name = models.CharField(max_length=90, db_column='tns_name')
    ra = models.FloatField(db_column='ra')
    dec = models.FloatField(db_column='dec')
    expname = models.CharField(max_length=90, db_column='expname')
    mag = models.FloatField(db_column='mag')
    dm = models.FloatField(db_column='dm')
    filter = models.CharField(max_length=90, db_column='filter')
    mjd = models.FloatField(db_column='mjd')

    class Meta:
        """Meta.
        """

        managed = False
        db_table = 'atlas_v_recurrencesddc_pessto'



# Functions within this file are too Django-specific to include in 'utils'.

# ATLAS version of the Django Search Form Helper Function.

import sys
from atlas.models import AtlasDiffObjects, AtlasDetectionsddc, AtlasDiffDetections
from atlas.models import TcsImages, TcsLatestObjectStats
from gkutils.commonutils import getObjectNamePortion, getCoordsAndSearchRadius, coneSearchHTM, FULL, dbConnect, CAT_ID_RA_DEC_COLS, ra_to_sex, dec_to_sex, Struct
from django.db.models import Q    # Need Q objects for OR query
import socket
from .lightcurvequeries import getLCData, getRecurrenceDataForPlotting

SHOW_LC_DATA_LIMIT = 50

def processSearchForm(searchText, getAssociatedData = False, ddc = False, getNonDets = False, getNearbyObjects = False):
    """processSearchForm.

    Args:
        searchText:
        getAssociatedData:
        ddc:
        getNonDets:
        getNearbyObjects:
    """
    from django.db import connection

    results = []
    # Is it a name?
    name = getObjectNamePortion(searchText)
    if name:
        #name = name.replace(' ','')
        # Do a name search
        if name.startswith('20'):
            results = AtlasDiffObjects.objects.filter(Q(other_designation__isnull = False) & Q(other_designation__startswith = name))
            # No results? Try the external crossmatches table.
            if len(results) == 0:
                q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
                results = list(set([x.id for x in q]))
        elif name.startswith('ATLAS'):
            results = AtlasDiffObjects.objects.filter(Q(atlas_designation__isnull = False) & Q(atlas_designation__startswith = name))
            # No results? Try the external crossmatches table.
            if len(results) == 0:
                q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
                results = list(set([x.id for x in q]))
        elif name.startswith('AT'):
            results = AtlasDiffObjects.objects.filter(Q(other_designation__isnull = False) & Q(other_designation__startswith = name.replace('AT','')))
            # No results? Try the external crossmatches table.
            if len(results) == 0:
                q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
                results = list(set([x.id for x in q]))
        elif name.startswith('SN'):
            results = AtlasDiffObjects.objects.filter(Q(other_designation__isnull = False) & Q(other_designation__startswith = name.replace('SN','')))
            # No results? Try the external crossmatches table.
            if len(results) == 0:
                q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
                results = list(set([x.id for x in q]))
        else: # name.startswith('ZTF'):
            # Check for external crossmatches.
            q = TcsLatestObjectStats.objects.filter(external_crossmatches__isnull = False).filter(external_crossmatches__contains=name)
            results = list(set([x.id for x in q]))
        #else:
        #    results = AtlasDiffObjects.objects.filter(Q(atlas_designation__isnull = False) & (Q(atlas_designation__startswith = 'ATLAS' + name) | Q(other_designation__startswith = '20' + name)))
    else:
         # It must be a coordinate or a match failure
         coords = getCoordsAndSearchRadius(searchText)
         if coords:
             searchRadius = 4.0
             if coords['radius']:
                 searchRadius = float(coords['radius'])
                 if searchRadius > 99.0:
                      searchRadius = 99.0
         
             message, xmObjects = coneSearchHTM(coords['ra'], coords['dec'], searchRadius, 'atlas_diff_objects', queryType = FULL, conn = connection, django = True)
             for xm in xmObjects:
                 dictRow = xm[1]
                 dictRow.update({'xmseparation': xm[0]})
                 # Add sexagesimal RA and Dec
                 dictRow.update({'ra_sex': ra_to_sex(dictRow['ra'])}) 
                 dictRow.update({'dec_sex': dec_to_sex(dictRow['dec'])}) 
                 if dictRow['images_id']:
                     # Get images
                     images = TcsImages.objects.get(pk=dictRow['images_id'])
                     if images:
                         dictRow.update({'images_id': images})
                 # 2018-08-07 KWS Convert the class into a dict so we can use common code elsewhere.
                 results.append(Struct(**dictRow))

    return results


def getNearbyObjectsForScatterPlot(candidate, ra, dec, coneSearchRadius = 8.0, ddc = False):
    """getNearbyObjectsForScatterPlot.

    Args:
        candidate:
        ra:
        dec:
        coneSearchRadius:
        ddc:
    """
    from django.db import connection

    recurrencePlotData = []
    recurrencePlotLabels = []
    averageObjectCoords = []
    rmsScatter = []

    xmObjects = None

    # Grab all objects within 3 arcsec of this one.
    xmList = []
    catalogueName = 'atlas_diff_objects'
    message, xmObjects = coneSearchHTM(ra, dec, coneSearchRadius, catalogueName, queryType = FULL, conn = connection, django = True)

    # The crossmatch Objects xmObjects are a list of two entry lists. The first entry in each row is the separaion.
    # The second entry is the catalogue row dictionary listing the relevant crossmatch.

    if xmObjects:
        numberOfMatches = len(xmObjects)
        # Add the objects into a list of dicts that have consistent names for all catalogues

        for xm in xmObjects:
            sys.stderr.write("\n%s\n" % str(xm))
            # Add to the list all object ids except the current one (which will have
            # a separation of zero).
            if xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][0]] != candidate:
                xmList.append({'xmseparation': xm[0], 'xmid': xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][0]]})
                xmid = xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][0]]
                xmra = xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][1]]
                xmdec = xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][2]]
                xmName = xm[1]["atlas_designation"]
                if not xmName:
                    xmName = xmid

                xmRecurrences = []
                if ddc:
                    xmDetections = AtlasDetectionsddc.objects.filter(atlas_object_id = xmid)
                    for xmDet in xmDetections:
                        if xmDet.det != 5:
                            xmRecurrences.append({"RA": xmDet.ra, "DEC": xmDet.dec})
                    if len(xmRecurrences) == 0: 
                        xmRecurrences = []
                        for xmDet in xmDetections:
                            xmRecurrences.append({"RA": xmDet.ra, "DEC": xmDet.dec})
                else:
                    xmDetections = AtlasDiffDetections.objects.filter(atlas_object_id = xmid)
                    for xmDet in xmDetections:
                        xmRecurrences.append({"RA": xmDet.ra, "DEC": xmDet.dec})


                xmrecurrencePlotData, xmrecurrencePlotLabels, xmaverageObjectCoords, xmrmsScatter = getRecurrenceDataForPlotting(xmRecurrences, ra, dec, secRA = xmra, secDEC = xmdec, secId = xmid, secName = xmName, objectColour = 23)
                recurrencePlotData += xmrecurrencePlotData
                recurrencePlotLabels += xmrecurrencePlotLabels
                averageObjectCoords += xmaverageObjectCoords
                rmsScatter += xmrmsScatter

    recurrenceData = [recurrencePlotData, recurrencePlotLabels, averageObjectCoords, rmsScatter]
    return recurrenceData


def getNearbyObjectsFromAlternateDatabase(currentDBName, alternateDBConnections, transient, coneSearchRadius, catalogueName = 'atlas_diff_objects'):
    """getNearbyObjectsFromAlternateDatabase.

    Args:
        currentDBName:
        alternateDBConnections:
        transient:
        coneSearchRadius:
        catalogueName:
    """
    results = {}
    oldDBXmList = []

    oldDBURL = None
    try:
        # HARD WIRED OLD DATABASES DEPEND ON CURRENT DB NAME
        oldDB = alternateDBConnections[currentDBName]
        oldDBURL = oldDB[2]

        # We have to make a manual connection to the database
        try:
            conn = dbConnect(oldDB[1], 'kws', '', oldDB[0], quitOnError = False)

            message, xmObjects = coneSearchHTM(transient.ra, transient.dec, coneSearchRadius, catalogueName, queryType = FULL, conn = conn, django = False)
            conn.close()

            if xmObjects:
                for xm in xmObjects:
                    oldDBXmList.append({'xmseparation': xm[0], 'xmid': xm[1][CAT_ID_RA_DEC_COLS[catalogueName][0][0]], 'xmName': xm[1]["atlas_designation"], 'xmTNS': xm[1]["other_designation"], 'xmCounter': xm[1]["followup_counter"]})

        except Exception as e:
            # If for any reason we can't connect to the old database, just continue anyway
            sys.stderr.write("\n%s\n" % str(e))
            pass

    except KeyError as e:
        # If for any reason we can't find the old database in the dictionary, don't bother doing anything
        sys.stderr.write("\n%s\n" % str(e))
        pass

    xmNearestName = None
    xmNearestCounter = None

    if oldDBXmList and oldDBXmList[0]['xmName']:
        xmNearestName = oldDBXmList[0]['xmName']
        xmNearestCounter = oldDBXmList[0]['xmCounter']

    results['oldDBURL'] = oldDBURL
    results['oldDBXmList'] = oldDBXmList
    results['xmNearestName'] = xmNearestName
    results['xmNearestCounter'] = xmNearestCounter

    return results

# New code to send a message to a daemon listening on a socket.

TNS_MESSAGES = { 'SUBMIT': 'Submit',
                 'SUBMITTEST': 'SubmitTest',
                 'RESULTS': 'Results' }

def sendMessage(host, port, message):
    '''
    Send a message to a listener.
    '''

    response = None

    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((host, port))
        s.sendall(message.encode('utf-8'))
        response = s.recv(1024).decode('utf-8')
        s.close()
    except socket.error as e:
        sys.stderr.write("Cannot connect. Maybe the service has not been started or is listening on a different port.\n")
        sys.stderr.write("%s" % str(e))

    return response


def filterGetParameters(request, queryFilter, prefix = ''):
    """filterGetParameters.

    Args:
        request:
        queryFilter:
        prefix:
    """
    # When we're querying for atlas_diff_objects we probably have lots of
    # common things to pull out.  Hence this helper function. Prefix can
    # be used if we need to refer the parameters remotely, but note that it
    # applies to all parameters at the moment.

    # 2020-10-13 KWS Add ability to do greater than, greater than or equal to,
    #                less than, less than or equal to.
    import datetime
    flagDate = None
    flagParameter = 'followup_flag_date'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        flagParameter = 'followup_flag_date' + suffix
        flagDate = request.GET.get(flagParameter)
        if flagDate is not None:
            break

    try:
        flagDate = datetime.datetime.strptime(flagDate, "%Y-%m-%d").date()
    except ValueError as e:
        flagDate = None
    except TypeError as e:
        flagDate = None

    if flagDate:
        queryFilter[prefix + flagParameter] = flagDate

    # 2020-10-13 KWS Added confidence_factor (i.e. RB factor)
    zooniverseScore = None
    zooniverseScoreParameter = 'zooniverse_score'
    for suffix in ['__lt', '__lte', '__gt', '__gte', '']:
        zooniverseScoreParameter = 'zooniverse_score' + suffix
        zooniverseScore = request.GET.get(zooniverseScoreParameter)
        if zooniverseScore is not None:
            break

    try:
        zooniverseScore = float(zooniverseScore)
    except ValueError as e:
        zooniverseScore = None
    except TypeError as e:
        zooniverseScore = None

    if zooniverseScore:
        queryFilter[prefix + zooniverseScoreParameter] = zooniverseScore

    otherDesignation = None
    otherDesignationParameter = 'other_designation'
    for suffix in ['__startswith', '__endswith', '']:
        otherDesignationParameter = 'other_designation' + suffix
        otherDesignation = request.GET.get(otherDesignationParameter)
        if otherDesignation is not None:
            break

    if otherDesignation:
        queryFilter[prefix + otherDesignationParameter] = otherDesignation

    objectType = None
    objectType = request.GET.get('object_classification')
    try:
        objectType = int(objectType)
    except ValueError as e:
        objectType = None
    except TypeError as e:
        objectType = None

    if objectType:
        queryFilter[prefix + 'object_classification'] = objectType

    sherlockType = None
    sherlockType = request.GET.get('sherlockClassification')

    if sherlockType:
        queryFilter[prefix + 'sherlockClassification'] = sherlockType

    specType = None
    specType = request.GET.get('observation_status')

    if specType:
        queryFilter[prefix + 'observation_status'] = specType

    return queryFilter

def getDjangoTables2ImageTemplate(imageType):
    IMAGE_TEMPLATE = """<img id="stampimages" src="{{ MEDIA_URL }}images/data/{{ dbname }}/{{ record.images_id.whole_mjd }}/{{ record.images_id.%s }}.jpeg" alt="triplet" title="{{ record.images_id.pss_filename }}" onerror="this.src='{{ STATIC_URL }}images/image_not_available.jpeg';" height="200" />""" % imageType
    return IMAGE_TEMPLATE




from psdb.models import TcsCatalogueTables
from psdb.models import TcsTransientObjects
from django.contrib import admin

class TcsObjectAdmin(admin.ModelAdmin):
    """TcsObjectAdmin.
    """

    fieldsets = [
        (None,               {'fields': ['id']}),
        ('Follow-up information', {'fields': ['external_reference_id'], 'classes': ['collapse']}),
    ]
    list_display = ('id', 'tcs_cmf_metadata_id','tcs_images_id', 'date_inserted', 'date_modified', 'object_classification', 'followup_priority', 'external_reference_id')
    search_fields = ['id']
    date_hierarchy = 'date_inserted'

admin.site.register(TcsTransientObjects, TcsObjectAdmin)
admin.site.register(TcsCatalogueTables)


# 2012-07-18 KWS Pure raw SQL code that can be shared across scripts and web apps.
#                The downside is that there is a security risk with this kind of
#                query (SQL injection attack).  However, I got fed up of writing
#                the same query again and again.

#                DO NOT IMPORT ANY DJANGO CODE IN HERE!! Any required Django objects
#                should be passed in by the calling code (e.g. djangoRawObject).

# Detection limits for drawing non-detection arrows on plots.
GLIMIT = 23.6
RLIMIT = 23.6
ILIMIT = 23.6
ZLIMIT = 22.6
YLIMIT = 21.3

# Add new limits for other filters - e.g. w and V.

LIMITS = {"g": GLIMIT,
          "r": RLIMIT,
          "i": ILIMIT,
          "z": ZLIMIT,
          "y": YLIMIT}

# Thresholds for doing PS1 colour plots.
FILTER_RECURRENCE_PERIOD = 0.5 # i.e. 1 day
FILTER_RECURRENCE_PERIOD_GR = 0.5
FILTER_RECURRENCE_PERIOD_RI = 2.0
FILTER_RECURRENCE_PERIOD_IZ = 2.0

# 2016-11-30 KWS Set COUNTS_LIMIT arbitrarily to 200 before using calculated offset
COUNTS_LIMIT = 200

# Pure raw text queries.  Note that if used by Django, there must be an associated model
# created in dbviews. The model is passed in as a parameter if required.

LC_PLAIN_TEXT_QUERY = '''\
          select o.id, o.id transient_object_id, m.imageid, m.mjd_obs, o.ra_psf, o.dec_psf, o.psf_inst_mag, o.psf_inst_mag_sig, o.ap_mag, o.cal_psf_mag, o.psf_inst_flux, o.psf_inst_flux_sig, substr(m.fpa_filter,1,1) filter, o.flags, m.filename cmf_file, i.name
            from tcs_transient_objects o
      inner join tcs_cmf_metadata m
              on (o.tcs_cmf_metadata_id = m.id)
       left join tcs_image_groups i
              on (o.image_group_id = i.id)
           where o.id = %s
             and m.mjd_obs > %s
           union all
          select r.id, r.transient_object_id, m.imageid, m.mjd_obs, r.ra_psf, r.dec_psf, r.psf_inst_mag, r.psf_inst_mag_sig, r.ap_mag, r.cal_psf_mag, r.psf_inst_flux, r.psf_inst_flux_sig, substr(m.fpa_filter,1,1) filter, r.flags, m.filename cmf_file, i.name
            from tcs_transient_reobservations r
      inner join tcs_cmf_metadata m
              on (r.tcs_cmf_metadata_id = m.id)
       left join tcs_image_groups i
              on (r.image_group_id = i.id)
           where r.transient_object_id = %s
             and m.mjd_obs > %s
        order by mjd_obs desc'''


LC_POINTS_QUERY = """\
         select id, mag, magerr, mjd, filter, exptime, inst_mag
           from (
         select o.id, o.cal_psf_mag mag, o.psf_inst_mag_sig magerr, m.mjd_obs mjd, substr(m.fpa_filter,1,1) filter, m.exptime, o.psf_inst_mag inst_mag
           from tcs_transient_objects o, tcs_cmf_metadata m
          where o.tcs_cmf_metadata_id = m.id
            and o.id = %s
            and o.cal_psf_mag is not null
         union all
         select r.transient_object_id id, r.cal_psf_mag mag, r.psf_inst_mag_sig magerr, m.mjd_obs mjd, substr(m.fpa_filter,1,1) filter, m.exptime, r.psf_inst_mag inst_mag
           from tcs_transient_reobservations r, tcs_cmf_metadata m
          where r.tcs_cmf_metadata_id = m.id
            and r.transient_object_id = %s
            and r.cal_psf_mag is not null
         ) temp
         where (filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%'))
         order by mjd
      """

# 2014-11-05 KWS Could do with above again, but this time the flux, not mags
# 2015-10-12 KWS Added zero_pt.
LC_POINTS_FLUX_QUERY = """\
         select id, flux, fluxerr, mjd, filter, exptime, inst_mag, zero_pt
           from (
         select o.id, o.psf_inst_flux flux, o.psf_inst_flux_sig fluxerr, m.mjd_obs mjd, substr(m.fpa_filter,1,1) filter, m.exptime, o.psf_inst_mag inst_mag, m.zero_pt
           from tcs_transient_objects o, tcs_cmf_metadata m
          where o.tcs_cmf_metadata_id = m.id
            and o.id = %s
            and o.cal_psf_mag is not null
         union all
         select r.transient_object_id id, r.psf_inst_flux flux, r.psf_inst_flux_sig fluxerr, m.mjd_obs mjd, substr(m.fpa_filter,1,1) filter, m.exptime, r.psf_inst_mag inst_mag, m.zero_pt
           from tcs_transient_reobservations r, tcs_cmf_metadata m
          where r.tcs_cmf_metadata_id = m.id
            and r.transient_object_id = %s
            and r.cal_psf_mag is not null
         ) temp
         where (filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%'))
         order by mjd
      """


# 2016-11-30 KWS Pull out the deteff_counts, deteff_magref, deteff_calculated_offset,
#                exptime, zero_pt values so we can plot limiting mags
LC_BLANKS_QUERY = """\
         select id, mjd, filter, deteff_counts, deteff_magref, deteff_calculated_offset, exptime, zero_pt
           from (
         select o.id, o.cal_psf_mag mag, o.psf_inst_mag_sig magerr, m.mjd_obs mjd, substr(m.fpa_filter,1,1) filter, deteff_counts, deteff_magref, deteff_calculated_offset, exptime, zero_pt
           from tcs_transient_objects o, tcs_cmf_metadata m
          where o.tcs_cmf_metadata_id = m.id
            and o.id = %s
            and o.cal_psf_mag is null
         union all
         select r.transient_object_id id, r.cal_psf_mag mag, r.psf_inst_mag_sig magerr, m.mjd_obs mjd, substr(m.fpa_filter,1,1) filter, deteff_counts, deteff_magref, deteff_calculated_offset, exptime, zero_pt
           from tcs_transient_reobservations r, tcs_cmf_metadata m
          where r.tcs_cmf_metadata_id = m.id
            and r.transient_object_id = %s
            and r.cal_psf_mag is null
         ) temp
         where (filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%') or filter like concat(%s,'%%'))
         order by mjd
      """

# 2013-07-17 KWS Corrected query below to be compatible with BOTH 3pi and MD.
# 2014-01-08 KWS Introduced the new skycell and tessellation columns into the
#                query, which vastly improves the speed.
# 2016-11-30 KWS Pull out the deteff_counts, deteff_magref, deteff_calculated_offset,
#                exptime, zero_pt values so we can plot limiting mags
LC_NON_DET_QUERY = """\
         select distinct mjd_obs mjd,
                         substr(mm.fpa_filter,1,1) filter, 1 id, deteff_counts, deteff_magref, deteff_calculated_offset, exptime, zero_pt
           from tcs_cmf_metadata mm,
           (
             select distinct field, skycell
               from (
               select
                      skycell,
                      case
                          when instr(m.tessellation,'MD') then substr(m.tessellation, instr(m.tessellation,'MD'),4)
                          when instr(m.tessellation,'RINGS') then substr(m.tessellation, instr(m.tessellation,'RINGS'),8)
                          else 'null'
                      end as field
                 from tcs_cmf_metadata m, tcs_transient_objects o
                where o.tcs_cmf_metadata_id = m.id
                  and o.id = %s
                union
               select
                      skycell,
                      case
                          when instr(m.tessellation,'MD') then substr(m.tessellation, instr(m.tessellation,'MD'),4)
                          when instr(m.tessellation,'RINGS') then substr(m.tessellation, instr(m.tessellation,'RINGS'),8)
                          else 'null'
                      end as field
                 from tcs_cmf_metadata m, tcs_transient_reobservations r
                where r.tcs_cmf_metadata_id = m.id
                  and r.transient_object_id = %s
               ) fieldandskycell
           ) det
         where mm.skycell = det.skycell
           and mm.tessellation like concat(det.field,'%%')
           and imageid not in
             (
                      select imageid
                        from (
                      select m.imageid
                        from tcs_transient_objects o, tcs_cmf_metadata m
                       where o.tcs_cmf_metadata_id = m.id
                         and o.id = %s
                         and o.cal_psf_mag is null
                      union all
                      select m.imageid
                        from tcs_transient_reobservations r, tcs_cmf_metadata m
                       where r.tcs_cmf_metadata_id = m.id
                         and r.transient_object_id = %s
                         and r.cal_psf_mag is null
                      ) temp1
             )
           and imageid not in
             (
                      select imageid
                        from (
                      select m.imageid
                        from tcs_transient_objects o, tcs_cmf_metadata m
                       where o.tcs_cmf_metadata_id = m.id
                         and o.id = %s
                         and o.cal_psf_mag is not null
                      union all
                      select m.imageid
                        from tcs_transient_reobservations r, tcs_cmf_metadata m
                       where r.tcs_cmf_metadata_id = m.id
                         and r.transient_object_id = %s
                         and r.cal_psf_mag is not null
                      ) temp2
             )
           and (fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%'))
         order by mm.mjd_obs
      """

# 2013-10-11 KWS Rehash of LC detections query based on the Non-detections query below.
#                This allows us to request stamps based on new detections whilst not
#                overwriting the existing ones.
# 2015-05-31 KWS Added filename, ppsub_input, ppsub_reference to selection.
LC_DET_QUERY = """\
         select mjd_obs mjd, substr(fpa_filter,1,1) filter, imageid, cast(truncate(mjd_obs,3) as char) tdate, ipp_idet, ra_psf, dec_psf, o.id, filename, ppsub_input, ppsub_reference, m.skycell sc,
                case
                    when instr(m.filename,'MD') then substr(m.filename, instr(m.filename,'MD'),4)
                    when instr(m.filename,'RINGS') then substr(m.filename, instr(m.filename,'RINGS'),8)
                    else 'null'
                end as field,
                case
                    -- 3pi
                    when instr(m.filename,'WS') then if(instr(m.filename,'skycell'), replace(substr(m.filename, instr(m.filename,'skycell'),instr(m.filename,'.dif') - instr(m.filename,'skycell')), '.WS', ''), 'null')
                    -- MD
                    else if(instr(m.filename,'skycell'), replace(substr(m.filename, instr(m.filename,'skycell'),instr(m.filename,'.dif') - instr(m.filename,'skycell')), '.SS', ''), 'null')
                end as skycell
           from tcs_transient_objects o, tcs_cmf_metadata m
          where o.tcs_cmf_metadata_id = m.id
            and o.id = %s
            and o.cal_psf_mag is not null
            and (fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%'))
         union all
         select mjd_obs mjd, substr(fpa_filter,1,1) filter, imageid, cast(truncate(mjd_obs,3) as char) tdate, ipp_idet, ra_psf, dec_psf, transient_object_id id, filename, ppsub_input, ppsub_reference, m.skycell sc,
                case
                    when instr(m.filename,'MD') then substr(m.filename, instr(m.filename,'MD'),4)
                    when instr(m.filename,'RINGS') then substr(m.filename, instr(m.filename,'RINGS'),8)
                    else 'null'
                end as field,
                case
                    -- 3pi
                    when instr(m.filename,'WS') then if(instr(m.filename,'skycell'), replace(substr(m.filename, instr(m.filename,'skycell'),instr(m.filename,'.dif') - instr(m.filename,'skycell')), '.WS', ''), 'null')
                    -- MD
                    else if(instr(m.filename,'skycell'), replace(substr(m.filename, instr(m.filename,'skycell'),instr(m.filename,'.dif') - instr(m.filename,'skycell')), '.SS', ''), 'null')
                end as skycell
           from tcs_transient_reobservations r, tcs_cmf_metadata m
          where r.tcs_cmf_metadata_id = m.id
            and r.transient_object_id = %s
            and r.cal_psf_mag is not null
            and (fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%'))
       order by mjd, imageid, ipp_idet
      """


# 2013-09-16 KWS Non-detections AND Blanks so that we can request postage stamps for blank areas...
# 2015-05-31 KWS Added filename, ppsub_input, ppsub_reference to selection.
LC_NON_DET_AND_BLANKS_QUERY = """\
         select distinct mjd_obs mjd, substr(mm.fpa_filter,1,1) filter, imageid, cast(truncate(mm.mjd_obs,3) as char) tdate, field, det.skycell, filename, ppsub_input, ppsub_reference, mm.skycell sc, mm.zero_pt, mm.exptime, mm.deteff_counts, mm.deteff_magref, mm.deteff_calculated_offset
           from tcs_cmf_metadata mm,
           (
             select distinct field, skycell
               from (
               select
                      skycell,
                      case
                          when instr(m.tessellation,'MD') then substr(m.tessellation, instr(m.tessellation,'MD'),4)
                          when instr(m.tessellation,'RINGS') then substr(m.tessellation, instr(m.tessellation,'RINGS'),8)
                          else 'null'
                      end as field
                 from tcs_cmf_metadata m, tcs_transient_objects o
                where o.tcs_cmf_metadata_id = m.id
                  and o.id = %s
                union
               select
                      skycell,
                      case
                          when instr(m.tessellation,'MD') then substr(m.tessellation, instr(m.tessellation,'MD'),4)
                          when instr(m.tessellation,'RINGS') then substr(m.tessellation, instr(m.tessellation,'RINGS'),8)
                          else 'null'
                      end as field
                 from tcs_cmf_metadata m, tcs_transient_reobservations r
                where r.tcs_cmf_metadata_id = m.id
                  and r.transient_object_id = %s
               ) fieldandskycell
           ) det
         where mm.skycell = det.skycell and mm.tessellation like concat(det.field,'%%')
           and imageid not in
             (
                      select imageid
                        from (
                      select m.imageid
                        from tcs_transient_objects o, tcs_cmf_metadata m
                       where o.tcs_cmf_metadata_id = m.id
                         and o.id = %s
                         and o.cal_psf_mag is not null
                      union all
                      select m.imageid
                        from tcs_transient_reobservations r, tcs_cmf_metadata m
                       where r.tcs_cmf_metadata_id = m.id
                         and r.transient_object_id = %s
                         and r.cal_psf_mag is not null
                      ) temp2
             )
           and (fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%'))
         order by mm.mjd_obs
      """


# This is a query for ATLAS detections.  We need the input image, the template image and the diff image so
# we can cut stamps out of them.

# 2014-07-08 KWS Filters can be grizywxBV, so add more OR statements accordingly.
LC_DET_QUERY_ATLAS = """\
         select mjd_obs mjd, substr(fpa_filter,1,1) filter, imageid, cast(truncate(mjd_obs,3) as char) tdate, ipp_idet, ra_psf, dec_psf, o.id, x_psf, y_psf, m.diffim, m.inputim, m.templim
           from tcs_transient_objects o, tcs_cmf_metadata m
          where o.tcs_cmf_metadata_id = m.id
            and o.id = %s
            and o.cal_psf_mag is not null
            and (fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%'))
         union all
         select mjd_obs mjd, substr(fpa_filter,1,1) filter, imageid, cast(truncate(mjd_obs,3) as char) tdate, ipp_idet, ra_psf, dec_psf, transient_object_id id, x_psf, y_psf, m.diffim, m.inputim, m.templim
           from tcs_transient_reobservations r, tcs_cmf_metadata m
          where r.tcs_cmf_metadata_id = m.id
            and r.transient_object_id = %s
            and r.cal_psf_mag is not null
            and (fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%') or fpa_filter like concat(%s,'%%'))
       order by mjd, imageid, ipp_idet
      """


# We want to make this query generic, so create as many series as we have Telescopes + Instruments + Filters
# Each telescope name, instrument name and filter is regarded as a separate group for plotting and colour calculations.
# The query extracts ALL data, both LC points and Limits.  The code afterwards will group the series by filter
# ensuring that limits are also represented in the same colour.
FOLLOWUP_PHOTOMETRY_QUERY = """\
          select p.id, p.transient_object_id, p.mjd, p.mag, p.magerr, p.filter, t.name telescope_name, t.description telescope_description, i.name instrument_name, i.description instrument_description
            from tcs_followup_photometry p, tcs_followup_telescopes t, tcs_followup_telescope_instruments i
           where p.telescope_instrument_id = i.id
             and i.telescope_id = t.id
             and p.transient_object_id = %s
        order by telescope_name, instrument_name, p.filter, p.mjd
      """

# Get forced photometry only if the flux s/n ratio > 3 and limit to a minimum MJD.
FORCED_PHOTOMETRY_QUERY = """\
          select id, transient_object_id, mjd_obs, ra_psf, dec_psf, skycell, exptime, psf_inst_mag, psf_inst_mag_sig, cal_psf_mag, psf_inst_flux, psf_inst_flux_sig, filter, zero_pt, fpa_id
            from tcs_forced_photometry
           where transient_object_id = %s
             and mjd_obs > %s
             and psf_inst_flux_sig > 0
             and abs(psf_inst_flux / psf_inst_flux_sig) > 0.0
        order by filter, mjd_obs
      """


# In this query, we'll extract each plotting series and their labels in one go.
def followupPhotometryQuery(candidate, djangoRawObject = None, conn = None):
   """followupPhotometryQuery.

   Args:
       candidate:
       djangoRawObject:
       conn:
   """
   from gkutils.commonutils import Struct

   resultset = []

   if djangoRawObject:
      # Assume a Django query by default
      resultset = djangoRawObject.objects.raw(FOLLOWUP_PHOTOMETRY_QUERY, [candidate])
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (FOLLOWUP_PHOTOMETRY_QUERY, [candidate])
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  resultset.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   return resultset



def getFollowupPhotometry(candidate, djangoRawObject = None, conn = None):
    """Use the query followupPhotometryQuery but organise the data for plotting"""

    # We need to create a dictionary of filters which contain arrays of the data.
    # It's messy to do this from scratch, so defaultdict to the rescue...
    from collections import defaultdict

    recurrences = followupPhotometryQuery(candidate, djangoRawObject = djangoRawObject, conn = conn)

    # Each set of data is returned ordered by telescope, filter, mjd.  So should be easy to create new
    # points arrays.

    fullList = []
    fullListBlanks = []

    # We'll make the filter data a dictionary. The keys will also serve as the point labels.
    filterData = defaultdict(list)
    filterDataBlanks = defaultdict(list)


    for row in recurrences:
        # For the full list, we don't care about which filter or telescope the data came from
        if row.magerr:
            fullList.append([row.mjd, row.mag, row.magerr])
            filterData[row.telescope_name + ' ' + row.filter].append([row.mjd, row.mag, row.magerr])

        # Blank errors mean that this is a limit observation.  This criterion may change in the
        # future, so we'll need to be prepared to alter the code.
        if not row.magerr:
            fullListBlanks.append([row.mjd, row.mag])
            filterDataBlanks[row.telescope_name + ' ' + row.filter].append([row.mjd, row.mag])
 
    # The fullList and fullList blanks data is purely there to allow us to constrain x and y limits
    return filterData, filterDataBlanks, fullList, fullListBlanks


def forcedPhotometryQuery(candidate, djangoRawObject = None, conn = None):
   """This method exists to allow extraction of all the column data for display as raw data"""
   from gkutils.commonutils import Struct

   mjdLimit = 0.0
   resultset = []

   if djangoRawObject:
      # Assume a Django query by default
      resultset = djangoRawObject.objects.raw(FORCED_PHOTOMETRY_QUERY, [candidate, mjdLimit])
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (FORCED_PHOTOMETRY_QUERY, [candidate, mjdLimit])
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  resultset.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   return resultset


def getForcedPhotometry(candidate, djangoRawObject = None, conn = None, limits = LIMITS):
    """Use the query forcedPhotometryQuery but organise the data for plotting.
       Grab all the flux data while we're at it, so we can do both mags and flux."""

    from collections import defaultdict
    from gkutils.commonutils import fluxToMicroJansky

    recurrences = forcedPhotometryQuery(candidate, djangoRawObject = djangoRawObject, conn = conn)

    fullList = []
    fullListBlanks = []
    fullListFlux = []

    # We'll make the filter data a dictionary. The keys will also serve as the point labels.
    filterData = defaultdict(list)
    filterDataFlux = defaultdict(list)
    filterDataBlanks = defaultdict(list)


    for row in recurrences:
        # 2020-02-03 KWS Present the flux in microJanskys.

        fullListFlux.append([row.mjd_obs, fluxToMicroJansky(row.psf_inst_flux, row.exptime, row.zero_pt), fluxToMicroJansky(row.psf_inst_flux_sig, row.exptime, row.zero_pt)])
        filterDataFlux[row.filter].append([row.mjd_obs, fluxToMicroJansky(row.psf_inst_flux, row.exptime, row.zero_pt), fluxToMicroJansky(row.psf_inst_flux_sig, row.exptime, row.zero_pt)])

        # Arbitrarily cut off the cal_psf_mag at 5.0.  Anything brighter than about 10 is likely to be garbage.
        if row.psf_inst_mag_sig and row.cal_psf_mag and row.cal_psf_mag >= 5.0:
            fullList.append([row.mjd_obs, row.cal_psf_mag, row.psf_inst_mag_sig])
            filterData[row.filter].append([row.mjd_obs, row.cal_psf_mag, row.psf_inst_mag_sig])

        # At the moment we don't pick up the forced blanks, but we'll leave this code in
        # for the time being.  We may modify the query in future to feed the blanks.
        if not row.psf_inst_mag_sig or not row.cal_psf_mag or row.cal_psf_mag < 5.0:
            fullListBlanks.append([row.mjd_obs, limits[row.filter]])
            filterDataBlanks[row.filter].append([row.mjd_obs, limits[row.filter]])

    # The fullList and fullList blanks data is purely there to allow us to constrain x and y limits
    return filterData, filterDataBlanks, fullList, fullListBlanks, filterDataFlux, fullListFlux



def lightcurvePlainQuery(candidate, mjdLimit = 55347.0, djangoRawObject = None, conn = None):
   """lightcurvePlainQuery.

   Args:
       candidate:
       mjdLimit:
       djangoRawObject:
       conn:
   """
   from gkutils.commonutils import Struct

   recurrences = []

   if djangoRawObject:
      # Assume a Django query by default
      recurrences = djangoRawObject.objects.raw(LC_PLAIN_TEXT_QUERY, [candidate, mjdLimit, candidate, mjdLimit])
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (LC_PLAIN_TEXT_QUERY, [candidate, mjdLimit, candidate, mjdLimit])
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  recurrences.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   return recurrences


# 2015-10-13 KWS Plain text forced photometry query
def lightcurveForcedPlainQuery(candidate, mjdLimit = 55347.0, djangoRawObject = None, conn = None):
   """lightcurveForcedPlainQuery.

   Args:
       candidate:
       mjdLimit:
       djangoRawObject:
       conn:
   """
   from gkutils.commonutils import Struct

   recurrences = []

   if djangoRawObject:
      # Assume a Django query by default
      recurrences = djangoRawObject.objects.raw(FORCED_PHOTOMETRY_QUERY, [candidate, mjdLimit])
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (FORCED_PHOTOMETRY_QUERY, [candidate, mjdLimit])
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  recurrences.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   return recurrences


# 2011-08-15 KWS Added the fudge factor to be applied to old data because of cal_psf_mag issues.
# 2014-07-03 KWS Added other filters.
def applyFudgeFactor(row):
   """applyFudgeFactor.

   Args:
       row:
   """

   from math import log10

   mjd = row.mjd
   magerr = row.magerr
   mag = None

   if row.mjd > 55248.0 and row.mjd < 55260.0 and row.mag is not None and row.inst_mag is not None and abs(row.mag) > 0 and abs(row.inst_mag) > 0 and row.filter in 'grizy':
      mag = row.inst_mag+25.0+2.5*log10(row.exptime)
   elif row.mjd < 55260.0 and (row.mag == 0.0 or row.mag is None) and row.inst_mag is not None and abs(row.inst_mag) > 0 and row.filter in 'grizy':
      mag = row.inst_mag+25.0+2.5*log10(row.exptime)
   elif (row.mjd >= 55260.0 and (row.mag == 0.0 or row.mag is None) and row.inst_mag is not None and abs(row.inst_mag) > 0) or (row.mjd > 55260.0 and row.mag > 24.5 and row.inst_mag is not None and abs(row.inst_mag) > 0):
      if row.filter == 'g':
         mag = row.inst_mag+25.0+2.5*log10(row.exptime)+2*(24.58 + 2.5*log10(113) - 25 - 2.5*log10(row.exptime))
      elif row.filter == 'r':
         mag = row.inst_mag+25.0+2.5*log10(row.exptime)+2*(24.80 + 2.5*log10(113) - 25 - 2.5*log10(row.exptime))
      elif row.filter == 'i':
         mag = row.inst_mag+25.0+2.5*log10(row.exptime)+2*(24.74 + 2.5*log10(240) - 25 - 2.5*log10(row.exptime))
      elif row.filter == 'z':
         mag = row.inst_mag+25.0+2.5*log10(row.exptime)+2*(24.26 + 2.5*log10(240) - 25 - 2.5*log10(row.exptime))
      elif row.filter == 'y':
         mag = row.inst_mag+25.0+2.5*log10(row.exptime)+2*(23.41 + 2.5*log10(240) - 25 - 2.5*log10(row.exptime))
      else: # Don't do anything - particularly for any new filters
         mag = row.mag

   else:
      mag = row.mag

   return [mjd, mag, magerr]


# 2013-02-03 KWS Make the query code generic. Pass in the Django object if Django, or the
#                connection object if a standard MySQL query.
def getLightcurvePoints(candidate, filters="grizywxBV", applyFudge = False, djangoRawObject = None, conn = None):
   """getLightcurvePoints.

   Args:
       candidate:
       filters:
       applyFudge:
       djangoRawObject:
       conn:
   """

   from gkutils.commonutils import Struct

   recurrences = []

   if djangoRawObject:
      # Assume a Django query by default
      recurrences = djangoRawObject.objects.raw(LC_POINTS_QUERY, (candidate, candidate, filters[0], filters[1], filters[2], filters[3], filters[4], filters[5], filters[6], filters[7], filters[8]))
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (LC_POINTS_QUERY, (candidate, candidate, filters[0], filters[1], filters[2], filters[3], filters[4], filters[5], filters[6], filters[7], filters[8]))
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  recurrences.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   g = []
   r = []
   i = []
   z = []
   y = []
   w = []
   x = []
   B = []
   V = []
   fullList = []
   for row in recurrences:
      if applyFudge:
         fullList.append(applyFudgeFactor(row))
      else:
         fullList.append([row.mjd, row.mag, row.magerr])

      # 2013-09-25 KWS Reduced verbose repetative code by using eval.
      if row.filter in 'grizywxBV':
         if applyFudge:
            eval(row.filter).append(applyFudgeFactor(row))
         else:
            eval(row.filter).append([row.mjd, row.mag, row.magerr])

   return [g, r, i, z, y, w, x, B, V, fullList]

# 2014-11-05 KWS Flux query
def getLightcurvePointsFlux(candidate, filters="grizywxBV", applyFudge = False, djangoRawObject = None, conn = None):
   """getLightcurvePointsFlux.

   Args:
       candidate:
       filters:
       applyFudge:
       djangoRawObject:
       conn:
   """

   from gkutils.commonutils import Struct

   recurrences = []

   if djangoRawObject:
      # Assume a Django query by default
      recurrences = djangoRawObject.objects.raw(LC_POINTS_FLUX_QUERY, (candidate, candidate, filters[0], filters[1], filters[2], filters[3], filters[4], filters[5], filters[6], filters[7], filters[8]))
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (LC_POINTS_FLUX_QUERY, (candidate, candidate, filters[0], filters[1], filters[2], filters[3], filters[4], filters[5], filters[6], filters[7], filters[8]))
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  recurrences.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   g = []
   r = []
   i = []
   z = []
   y = []
   w = []
   x = []
   B = []
   V = []
   fullList = []
   for row in recurrences:
      if applyFudge:
         fullList.append(applyFudgeFactor(row))
      else:
         fullList.append([row.mjd, row.flux, row.fluxerr, row.exptime, row.zero_pt])

      # 2013-09-25 KWS Reduced verbose repetative code by using eval.
      if row.filter in 'grizywxBV':
         if applyFudge:
            eval(row.filter).append(applyFudgeFactor(row))
         else:
            eval(row.filter).append([row.mjd, row.flux, row.fluxerr, row.exptime, row.zero_pt])

   return [g, r, i, z, y, w, x, B, V, fullList]



def getLightcurveBlanks(candidate, filters="grizywxBV", djangoRawObject = None, conn = None, limits = LIMITS):
   """getLightcurveBlanks.

   Args:
       candidate:
       filters:
       djangoRawObject:
       conn:
       limits:
   """

   from gkutils.commonutils import Struct
   from math import log10

   recurrences = []

   if djangoRawObject:
      # Assume a Django query by default
      recurrences = djangoRawObject.objects.raw(LC_BLANKS_QUERY, (candidate, candidate, filters[0], filters[1], filters[2], filters[3], filters[4], filters[5], filters[6], filters[7], filters[8]))
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (LC_BLANKS_QUERY, (candidate, candidate, filters[0], filters[1], filters[2], filters[3], filters[4], filters[5], filters[6], filters[7], filters[8]))
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  recurrences.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []

   g = []
   r = []
   i = []
   z = []
   y = []
   w = []
   x = []
   B = []
   V = []
   fullList = []
   for row in recurrences:
      fullList.append([row.mjd])
      if row.filter in 'grizywxBV':
         if row.deteff_magref is None or row.deteff_counts is None or row.deteff_calculated_offset is None:
            eval(row.filter).append([float(row.mjd), limits[row.filter]])
         else:
            if int(row.deteff_counts) < COUNTS_LIMIT:
               detectionLimit = float(row.deteff_magref) + 2.5 * log10(row.exptime) + float(row.zero_pt)
            else:
               detectionLimit = float(row.deteff_magref) + 2.5 * log10(row.exptime) + float(row.zero_pt) + float(row.deteff_calculated_offset)
            eval(row.filter).append([float(row.mjd), detectionLimit])

   return [g, r, i, z, y, w, x, B, V, fullList]

def getLightcurveNonDetections(candidate, filters="grizywxBV", djangoRawObject = None, conn = None, limits = LIMITS):
   """getLightcurveNonDetections.

   Args:
       candidate:
       filters:
       djangoRawObject:
       conn:
       limits:
   """

   from gkutils.commonutils import Struct
   from math import log10

   recurrences = []

   if djangoRawObject:
      # Assume a Django query by default
      recurrences = djangoRawObject.objects.raw(LC_NON_DET_QUERY, (candidate, candidate, candidate, candidate, candidate, candidate, filters[0], filters[1], filters[2], filters[3], filters[4], filters[5], filters[6], filters[7], filters[8]))
   elif conn:
      # Otherwise try a raw MySQL query if we have a connecton object
      import MySQLdb
      try:
          cursor = conn.cursor(MySQLdb.cursors.DictCursor)
          cursor.execute (LC_NON_DET_QUERY, (candidate, candidate, candidate, candidate, candidate, candidate, filters[0], filters[1], filters[2], filters[3], filters[4], filters[5], filters[6], filters[7], filters[8]))
          results = cursor.fetchall ()
          cursor.close ()
          if results:
              for result in results:
                  recurrences.append(Struct(**result))

      except MySQLdb.Error as e:
          print("Error %d: %s" % (e.args[0], e.args[1]))
          return []
   else:
      return []


   g = []
   r = []
   i = []
   z = []
   y = []
   w = []
   x = []
   B = []
   V = []
   fullList = []
   for row in recurrences:
      fullList.append([float(row.mjd)])
      if row.filter in 'grizywxBV':
         if row.deteff_magref is None or row.deteff_counts is None or row.deteff_calculated_offset is None:
            eval(row.filter).append([float(row.mjd), limits[row.filter]])
         else:
            if int(row.deteff_counts) < COUNTS_LIMIT:
               detectionLimit = float(row.deteff_magref) + 2.5 * log10(row.exptime) + float(row.zero_pt)
            else:
               detectionLimit = float(row.deteff_magref) + 2.5 * log10(row.exptime) + float(row.zero_pt) + float(row.deteff_calculated_offset)
            eval(row.filter).append([float(row.mjd), detectionLimit])

   return [g, r, i, z, y, w, x, B, V, fullList]



def getLimitsFromLCData(dataList):
    '''Get the xMax, xMin, yMax, yMin of provided data'''

    # Should probably move this into utils

    # At some stage we also want to provide a mechanism for getting
    # current MJD and current MJD - 90 days...

    import numpy as n

    # This code assumes that all the data is in the form of x, y, error.
    # The dataList elements are ususally the grizy merged array, the
    # grizy blanks and grizy non-detections.

    # If no error is provided, a zero error column will be added.

    # Want to return individual array max/mins in a list, and total max/min

    xMin = []
    yMin = []
    xMax = []
    yMax = []

    firstPass = True

    for row in dataList:

        if not row:
            continue

        dataArray = n.array(row)

        # If there's only ONE column:
        if len(row[0]) < 2:
            nans = []
            for i in range(len(row)):
                nans.append([n.nan])
            dataArray = n.column_stack((dataArray, n.array(nans)) )

        # If there's only TWO columns:
        if len(row[0]) < 3:
            dataArray = n.column_stack((dataArray, n.zeros( (len(row),1) )))

        if firstPass:
            concatArray = dataArray
        else:
            concatArray = n.vstack((dataArray, concatArray))

        firstPass = False

        xMin.append(n.nanmin(dataArray[:,0]))
        xMax.append(n.nanmax(dataArray[:,0]))
        yMin.append(n.nanmin(dataArray[:,1]))
        yMax.append(n.nanmax(dataArray[:,1]))

    if not firstPass:
        # If the code above has run, then firstPass must always be false.
        xMin.append(n.nanmin(concatArray[:,0]))
        xMax.append(n.nanmax(concatArray[:,0]))
        yMin.append(n.nanmin(concatArray[:,1]))
        yMax.append(n.nanmax(concatArray[:,1]))

    # Return 4 arrays whose length is 1+ number of input arrays.
    # The last element of each array is the min/max for the concatenated
    # arrays.

    return xMin, xMax, yMin, yMax


def colourDataPlainQuery(candidate, applyFudge = False, djangoRawObject = None, conn = None):
    """Call the previously defined queries to return plain text colour data"""

    from gkutils.commonutils import getColour, getColourStats

    # All the code to do this query is defined above, so it should be a simple matter
    # of calling that code.

    # Although we request the y-band data, we'll not bother dealing with z-y for the time being.

    lcData = getLightcurvePoints(candidate, applyFudge = applyFudge, djangoRawObject = djangoRawObject, conn = conn)

    # For the colour plots (later) list the griz data separately
    g = lcData[0]
    r = lcData[1]
    i = lcData[2]
    z = lcData[3]
    y = lcData[4]

    grColour = []
    riColour = []
    izColour = []

    meangr = None
    meanri = None
    meaniz = None

    grEvolution = None
    riEvolution = None
    izEvolution = None

    if g and r:
        grColour = getColour(g, r, FILTER_RECURRENCE_PERIOD_GR)
        if grColour and len(grColour) > 1:
            meangr, grEvolution = getColourStats(grColour)
    if r and i:
        riColour = getColour(r, i, FILTER_RECURRENCE_PERIOD_RI)
        if riColour and len(riColour) > 1:
            meanri, riEvolution = getColourStats(riColour)
    if i and z:
        izColour = getColour(i, z, FILTER_RECURRENCE_PERIOD_IZ)
        if izColour and len(izColour) > 1:
            meaniz, izEvolution = getColourStats(izColour)

    return grColour, meangr, grEvolution, riColour, meanri, riEvolution, izColour, meaniz, izEvolution


def getRecurrenceData(candidate, djangoRawObject = None, conn = None):
    """Get all the recurrence plotting data for dynamic recurrence plots"""

    # Create a dict of RA and DEC so that we can call the RMS function.
    # Also, pick out the designated object RA and DEC primary coords.

    from gkutils.commonutils import calculateRMSScatter

    recurrences = lightcurvePlainQuery(candidate, mjdLimit = 0, djangoRawObject = djangoRawObject, conn = conn)

    objectCoordinateRecurrences = []
    primaryObjectCoords = {}
    averageObjectCoords = {}

    primaryDetectionFound = False

    for row in recurrences:
        objectCoordinateRecurrences.append({"RA": row.RA, "DEC": row.DEC})

    avgRa, avgDec, rmsScatter =  calculateRMSScatter(objectCoordinateRecurrences)
    averageObjectCoords["RA"] = avgRa
    averageObjectCoords["DEC"] = avgDec

    return recurrences, averageObjectCoords, rmsScatter




from setuptools import setup, find_packages
import os

moduleDirectory = os.path.dirname(os.path.realpath(__file__))

def readme():
    with open(moduleDirectory + '/README.md') as f:
        return f.read()

setup(
    name="psat-server-web",
    description='Pan-STARRS and ATLAS web interface.',
    long_description=readme(),
    long_description_content_type="text/markdown",
    version="0.0.15",
    author='genghisken',
    author_email='ken.w.smith@gmail.com',
    license='MIT',
    url='https://github.com/genghisken/psat-server-web',
    packages=find_packages(),
    classifiers=[
          'Development Status :: 4 - Beta',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3.6',
          'Topic :: Utilities',
    ],
    install_requires=[
          'numpy',
          'mysqlclient',
          'django',
          'django_tables2',
          'pyyaml',
          'docopt',
          'python-dotenv',
          'gkhtm',
          'gkutils',
      ],
    python_requires='>=3.6',
    include_package_data=True,
    zip_safe=False
)
